@@ -31,13 +31,13 @@ my $oRenderTag =
 {
     'markdown' =>
     {
+        'br' => ['\n', ''],
         'quote' => ['"', '"'],
         'b' => ['**', '**'],
         'i' => ['_', '_'],
         # 'bi' => ['_**', '**_'],
-        'ul' => ["\n", ""],
-        'ol' => ["\n", "\n"],
-        'li' => ['- ', "\n"],
+        'list-item' => ["\n", ""],
+        'list-item' => ['- ', "\n"],
         'id' => ['`', '`'],
         'file' => ['`', '`'],
         'path' => ['`', '`'],
@@ -56,13 +56,14 @@ my $oRenderTag =
 
     'text' =>
     {
+        'br' => ['\n', ''],
         'quote' => ['"', '"'],
+        'p' => ['', "\n\n"],
         'b' => ['', ''],
         'i' => ['', ''],
         # 'bi' => ['', ''],
-        'ul' => ["\n", "\n"],
-        'ol' => ["\n", "\n"],
-        'li' => ['* ', "\n"],
+        'list' => ["", "\n"],
+        'list-item' => ['* ', "\n"],
         'id' => ['', ''],
         'host' => ['', ''],
         'file' => ['', ''],
@@ -78,18 +79,19 @@ my $oRenderTag =
         'backrest' => [undef, ''],
         'proper' => ['', ''],
         'postgres' => ['PostgreSQL', ''],
-        'admonition' => ["\n", "\n\n"],
+        'admonition' => ['', "\n\n"],
     },
 
     'latex' =>
     {
+        'br' => ['\\\vspace{1em}', ''],
         'quote' => ['``', '"'],
+        'p' => ["\n\\begin{sloppypar}", "\\end{sloppypar}\n"],
         'b' => ['\textbf{', '}'],
         'i' => ['\textit{', '}'],
         # 'bi' => ['', ''],
-        'ul' => ["\\begin{itemize}\n", "\\end{itemize}\n"],
-        # 'ol' => ["\n", "\n"],
-        'li' => ['\item ', "\n"],
+        'list' => ["\\begin{itemize}\n", "\\end{itemize}\n"],
+        'list-item' => ['\item ', "\n"],
         'id' => ['\textnormal{\texttt{', '}}'],
         'host' => ['\textnormal{\textbf{', '}}'],
         'file' => ['\textnormal{\texttt{', '}}'],
@@ -110,18 +112,20 @@ my $oRenderTag =
         'backrest' => [undef, ''],
         'proper' => ['\textnormal{\texttt{', '}}'],
         'postgres' => ['PostgreSQL', ''],
-        'admonition' => ["\n\\begin{leftbar}\n\\textit{\\textbf{", "}\n\\end{leftbar}\n"],
+        'admonition' =>
+            ["\n\\vspace{.5em}\\begin{leftbar}\n\\begin{sloppypar}\\textit{\\textbf{", "}\\end{sloppypar}\n\\end{leftbar}\n"],
     },
 
     'html' =>
     {
+        'br' => ['<br/>', ''],
         'quote' => ['<q>', '</q>'],
         'b' => ['<b>', '</b>'],
         'i' => ['<i>', '</i>'],
+        'p' => ['', ''],
         # 'bi' => ['<i><b>', '</b></i>'],
-        'ul' => ['<ul>', '</ul>'],
-        'ol' => ['<ol>', '</ol>'],
-        'li' => ['<li>', '</li>'],
+        'list' => ['<ul class="list-unordered">', '</ul>'],
+        'list-item' => ['<li class="list-unordered">', '</li>'],
         'id' => ['<span class="id">', '</span>'],
         'host' => ['<span class="host">', '</span>'],
         'file' => ['<span class="file">', '</span>'],
@@ -193,14 +197,14 @@ sub new
         my $oRenderOut =
             $self->{oManifest}->renderOutGet($self->{strType} eq 'latex' ? 'pdf' : $self->{strType}, $self->{strRenderOutKey});
 
-        # If these are the backrest docs then load the reference
+        # If these are the backrest docs then load the help
         if ($self->{oManifest}->isBackRest())
         {
             $self->{oReference} =
-                new pgBackRestDoc::Common::DocConfig(${$self->{oManifest}->sourceGet('reference')}{doc}, $self);
+                new pgBackRestDoc::Common::DocConfig(${$self->{oManifest}->sourceGet('help')}{doc}, $self);
         }
 
-        if (defined($$oRenderOut{source}) && $$oRenderOut{source} eq 'reference' && $self->{oManifest}->isBackRest())
+        if (defined($$oRenderOut{source}) && $$oRenderOut{source} eq 'help' && $self->{oManifest}->isBackRest())
         {
             if ($self->{strRenderOutKey} eq 'configuration')
             {
@@ -471,7 +475,7 @@ sub build
             $oNode->paramSet('depend-default', $strDependPrev);
         }
 
-        # Set log to true if this section has an execute list.  This helps reduce the info logging by only showing sections that are
+        # Set log to true if this section has an execute list. This helps reduce the info logging by only showing sections that are
         # likely to take a log time.
         $oNode->paramSet('log', $self->{bExe} && $oNode->nodeList('execute-list', false) > 0 ? true : false);
 
@@ -729,6 +733,14 @@ sub processTag
         if (!defined($strUrl))
         {
             my $strPage = $self->variableReplace($oTag->paramGet('page', false));
+            my $strSection = $oTag->paramGet('section', false);
+
+            # If a page/section link points to the current page then remove the page portion
+            if (defined($strPage) && defined($strSection) && defined($self->{strRenderOutKey}) &&
+                $strPage eq $self->{strRenderOutKey})
+            {
+                undef($strPage);
+            }
 
             # If this is a page URL
             if (defined($strPage))
@@ -741,11 +753,20 @@ sub processTag
                 # Else point locally
                 else
                 {
-                    if ($strType eq 'html' || $strType eq 'markdown')
+                    if ($strType eq 'html')
                     {
-                        $strUrl =
-                            $oTag->paramGet('page', false) . '.' .
-                            ($strType eq 'html' ? $strType : '.md');
+                        $strUrl = "${strPage}.html". (defined($strSection) ? '#' . substr($strSection, 1) : '');
+                    }
+                    elsif ($strType eq 'markdown')
+                    {
+                        if (defined($strSection))
+                        {
+                            confess &log(
+                                ERROR,
+                                "page and section links not supported for type ${strType}, value '" . $oTag->valueGet() . "'");
+                        }
+
+                        $strUrl = "${strPage}.md";
                     }
                     else
                     {
@@ -805,6 +826,10 @@ sub processTag
                 $strBuffer = "\\hyperref[$strUrl]{" . $oTag->valueGet() . "}";
             }
         }
+        elsif ($strType eq 'text')
+        {
+            $strBuffer = $oTag->valueGet();
+        }
         else
         {
             confess "'link' tag not valid for type ${strType}";
@@ -822,14 +847,14 @@ sub processTag
 
         $strBuffer .= $strStart;
 
-        # Admonitions in the reference materials are tags of the text element rather than field elements of the document so special
+        # Admonitions in the help materials are tags of the text element rather than field elements of the document so special
         # handling is required
         if ($strTag eq 'admonition')
         {
             $strBuffer .= $self->processAdmonitionStart($oTag);
         }
 
-        if ($strTag eq 'p' || $strTag eq 'title' || $strTag eq 'li' || $strTag eq 'code-block' || $strTag eq 'summary'
+        if ($strTag eq 'p' || $strTag eq 'title' || $strTag eq 'list-item' || $strTag eq 'code-block' || $strTag eq 'summary'
             || $strTag eq 'admonition')
         {
             $strBuffer .= $self->processText($oTag);
@@ -965,6 +990,7 @@ sub processText
 
     my $strType = $self->{strType};
     my $strBuffer = '';
+    my $strLastTag = 'body';
 
     foreach my $oNode ($oText->nodeList(undef, false))
     {
@@ -975,11 +1001,30 @@ sub processText
                 confess &log(ERROR, "unable to process quotes in string (use <quote> instead):\n${oNode}");
             }
 
-            $strBuffer .= $oNode;
+            # Skip text nodes with linefeeds since they happen between tags
+            if (index($oNode, "\n") == -1)
+            {
+                $strBuffer .= $oNode;
+            }
         }
         else
         {
+            # Add br tags to separate paragraphs and linefeeds to make the output more diffable. This is needed because of the hacky
+            # way config text is being rendered in the final document, i.e. by passing rendered HTML into divs rather than XML to be
+            # rendered at that time.
+            if ($strLastTag eq 'p' && $strType eq 'html')
+            {
+                $strBuffer .= "<br/>\n";
+
+                if ($oNode->nameGet() eq 'p')
+                {
+                    $strBuffer .= "<br/>\n";
+                }
+            }
+
             $strBuffer .= $self->processTag($oNode);
+
+            $strLastTag = $oNode->nameGet();
         }
     }
     #
@@ -1034,6 +1079,11 @@ sub processText
         }
     }
 
+    if ($strType eq 'html')
+    {
+        $strBuffer =~ s/ma[s]ter/ma&#115;ter/g;
+    }
+
     if ($strType eq 'text')
     {
         $strBuffer =~ s/\&mdash\;/--/g;
