@@ -35,6 +35,8 @@ Constants
 #define LOCK_KEY_EXEC_ID                                            STRID6("execId", 0x12e0c56051)
 #define LOCK_KEY_PERCENT_COMPLETE                                   STRID6("pctCplt", 0x14310a140d01)
 #define LOCK_KEY_PROCESS_ID                                         STRID5("pid", 0x11300)
+#define LOCK_KEY_SIZE_COMPLETE                                      STRID6("szCplt", 0x50c4286931)
+#define LOCK_KEY_SIZE                                               STRID5("sz", 0x3530)
 
 /***********************************************************************************************************************************
 Lock type names
@@ -152,6 +154,12 @@ lockReadFileData(const String *const lockFile, const int fd)
                     result.percentComplete = varNewUInt(jsonReadUInt(json));
 
                 result.processId = jsonReadInt(jsonReadKeyRequireStrId(json, LOCK_KEY_PROCESS_ID));
+
+                if (jsonReadKeyExpectStrId(json, LOCK_KEY_SIZE))
+                    result.size = varNewUInt64(jsonReadUInt(json));
+
+                if (jsonReadKeyExpectStrId(json, LOCK_KEY_SIZE_COMPLETE))
+                    result.sizeComplete = varNewUInt64(jsonReadUInt(json));
             }
             MEM_CONTEXT_PRIOR_END();
         }
@@ -266,6 +274,8 @@ lockWriteData(const LockType lockType, const LockWriteDataParam param)
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(ENUM, lockType);
         FUNCTION_LOG_PARAM(VARIANT, param.percentComplete);
+        FUNCTION_LOG_PARAM(VARIANT, param.sizeComplete);
+        FUNCTION_LOG_PARAM(VARIANT, param.size);
     FUNCTION_LOG_END();
 
     ASSERT(lockType < lockTypeAll);
@@ -285,6 +295,12 @@ lockWriteData(const LockType lockType, const LockWriteDataParam param)
 
         jsonWriteInt(jsonWriteKeyStrId(json, LOCK_KEY_PROCESS_ID), getpid());
 
+        if (param.size != NULL)
+            jsonWriteUInt64(jsonWriteKeyStrId(json, LOCK_KEY_SIZE), varUInt64(param.size));
+
+        if (param.sizeComplete != NULL)
+            jsonWriteUInt64(jsonWriteKeyStrId(json, LOCK_KEY_SIZE_COMPLETE), varUInt64(param.sizeComplete));
+
         jsonWriteObjectEnd(json);
 
         if (lockType == lockTypeBackup && lockLocal.held)
