@@ -37,7 +37,7 @@ struct Db
 };
 
 /***********************************************************************************************************************************
-Close protocol connection.  No need to close a locally created PgClient since it has its own destructor.
+Close protocol connection. No need to close a locally created PgClient since it has its own destructor.
 ***********************************************************************************************************************************/
 static void
 dbFreeResource(THIS_VOID)
@@ -54,12 +54,13 @@ dbFreeResource(THIS_VOID)
     pckWriteU32P(protocolCommandParam(command), this->remoteIdx);
 
     protocolClientExecute(this->remoteClient, command, false);
+    protocolCommandFree(command);
 
     FUNCTION_LOG_RETURN_VOID();
 }
 
 /**********************************************************************************************************************************/
-Db *
+FN_EXTERN Db *
 dbNew(PgClient *client, ProtocolClient *remoteClient, const Storage *const storage, const String *applicationName)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -73,12 +74,8 @@ dbNew(PgClient *client, ProtocolClient *remoteClient, const Storage *const stora
     ASSERT(storage != NULL);
     ASSERT(applicationName != NULL);
 
-    Db *this = NULL;
-
-    OBJ_NEW_BEGIN(Db)
+    OBJ_NEW_BEGIN(Db, .childQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1)
     {
-        this = OBJ_NEW_ALLOC();
-
         *this = (Db)
         {
             .pub =
@@ -100,35 +97,48 @@ dbNew(PgClient *client, ProtocolClient *remoteClient, const Storage *const stora
 /***********************************************************************************************************************************
 Execute a query
 ***********************************************************************************************************************************/
-static VariantList *
-dbQuery(Db *this, const String *query)
+static Pack *
+dbQuery(Db *this, const PgClientQueryResult resultType, const String *const query)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(DB, this);
+        FUNCTION_LOG_PARAM(STRING_ID, resultType);
         FUNCTION_LOG_PARAM(STRING, query);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
+    ASSERT(resultType != 0);
     ASSERT(query != NULL);
 
-    VariantList *result = NULL;
+    Pack *result = NULL;
 
     // Query remotely
     if (this->remoteClient != NULL)
     {
-        ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_DB_QUERY);
-        PackWrite *const param = protocolCommandParam(command);
+        MEM_CONTEXT_TEMP_BEGIN()
+        {
+            ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_DB_QUERY);
+            PackWrite *const param = protocolCommandParam(command);
 
-        pckWriteU32P(param, this->remoteIdx);
-        pckWriteStrP(param, query);
+            pckWriteU32P(param, this->remoteIdx);
+            pckWriteStrIdP(param, resultType);
+            pckWriteStrP(param, query);
 
-        result = varVarLst(jsonToVar(pckReadStrP(protocolClientExecute(this->remoteClient, command, true))));
+            PackRead *const read = protocolClientExecute(this->remoteClient, command, true);
+
+            MEM_CONTEXT_PRIOR_BEGIN()
+            {
+                result = pckReadPackP(read);
+            }
+            MEM_CONTEXT_PRIOR_END();
+        }
+        MEM_CONTEXT_TEMP_END();
     }
     // Else locally
     else
-        result = pgClientQuery(this->client, query);
+        result = pgClientQuery(this->client, query, resultType);
 
-    FUNCTION_LOG_RETURN(VARIANT_LIST, result);
+    FUNCTION_LOG_RETURN(PACK, result);
 }
 
 /***********************************************************************************************************************************
@@ -145,7 +155,7 @@ dbExec(Db *this, const String *command)
     ASSERT(this != NULL);
     ASSERT(command != NULL);
 
-    CHECK(AssertError, dbQuery(this, command) == NULL, "exec returned data");
+    CHECK(AssertError, dbQuery(this, pgClientQueryResultNone, command) == NULL, "exec returned data");
 
     FUNCTION_LOG_RETURN_VOID();
 }
@@ -153,8 +163,8 @@ dbExec(Db *this, const String *command)
 /***********************************************************************************************************************************
 Execute a query that returns a single row and column
 ***********************************************************************************************************************************/
-static Variant *
-dbQueryColumn(Db *this, const String *query)
+static PackRead *
+dbQueryColumn(Db *const this, const String *const query)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(DB, this);
@@ -164,19 +174,18 @@ dbQueryColumn(Db *this, const String *query)
     ASSERT(this != NULL);
     ASSERT(query != NULL);
 
-    VariantList *result = dbQuery(this, query);
+    Pack *const pack = dbQuery(this, pgClientQueryResultColumn, query);
+    PackRead *const result = pckReadNew(pack);
+    pckMove(pack, objMemContext(result));
 
-    CHECK(AssertError, varLstSize(result) == 1, "query must return one column");
-    CHECK(AssertError, varLstSize(varVarLst(varLstGet(result, 0))) == 1, "query must return one row");
-
-    FUNCTION_LOG_RETURN(VARIANT, varLstGet(varVarLst(varLstGet(result, 0)), 0));
+    FUNCTION_LOG_RETURN(PACK_READ, result);
 }
 
 /***********************************************************************************************************************************
 Execute a query that returns a single row
 ***********************************************************************************************************************************/
-static VariantList *
-dbQueryRow(Db *this, const String *query)
+static PackRead *
+dbQueryRow(Db *const this, const String *const query)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(DB, this);
@@ -186,11 +195,11 @@ dbQueryRow(Db *this, const String *query)
     ASSERT(this != NULL);
     ASSERT(query != NULL);
 
-    VariantList *result = dbQuery(this, query);
+    Pack *const pack = dbQuery(this, pgClientQueryResultRow, query);
+    PackRead *const result = pckReadNew(pack);
+    pckMove(pack, objMemContext(result));
 
-    CHECK(AssertError, varLstSize(result) == 1, "query must return one row");
-
-    FUNCTION_LOG_RETURN(VARIANT_LIST, varVarLst(varLstGet(result, 0)));
+    FUNCTION_LOG_RETURN(PACK_READ, result);
 }
 
 /***********************************************************************************************************************************
@@ -205,16 +214,19 @@ dbIsInRecovery(Db *const this)
 
     ASSERT(this != NULL);
 
-    bool result = false;
+    bool result;
 
-    if (dbPgVersion(this) >= PG_VERSION_HOT_STANDBY)
-        result = varBool(dbQueryColumn(this, STRDEF("select pg_catalog.pg_is_in_recovery()")));
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        result = pckReadBoolP(dbQueryColumn(this, STRDEF("select pg_catalog.pg_is_in_recovery()")));
+    }
+    MEM_CONTEXT_TEMP_END();
 
     FUNCTION_LOG_RETURN(BOOL, result);
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 dbOpen(Db *this)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -244,16 +256,16 @@ dbOpen(Db *this)
             this->pub.dbTimeout = cfgOptionUInt64(cfgOptDbTimeout);
         }
 
-        // Set search_path to prevent overrides of the functions we expect to call.  All queries should also be schema-qualified,
+        // Set search_path to prevent overrides of the functions we expect to call. All queries should also be schema-qualified,
         // but this is an extra level protection.
         dbExec(this, STRDEF("set search_path = 'pg_catalog'"));
 
-        // Set client encoding to UTF8.  This is the only encoding (other than ASCII) that we can safely work with.
+        // Set client encoding to UTF8. This is the only encoding (other than ASCII) that we can safely work with.
         dbExec(this, STRDEF("set client_encoding = 'UTF8'"));
 
-        // Query the version and data_directory
-        VariantList *row = dbQueryRow(
-            this,
+        // Query the version and data_directory. Be sure the update the total in the null check below when adding/removing columns.
+        const Pack *const row = dbQuery(
+            this, pgClientQueryResultRow,
             STRDEF(
                 "select (select setting from pg_catalog.pg_settings where name = 'server_version_num')::int4,"
                 " (select setting from pg_catalog.pg_settings where name = 'data_directory')::text,"
@@ -262,31 +274,36 @@ dbOpen(Db *this)
                 " (select setting from pg_catalog.pg_settings where name = 'checkpoint_timeout')::int4"));
 
         // Check that none of the return values are null, which indicates the user cannot select some rows in pg_settings
-        for (unsigned int columnIdx = 0; columnIdx < varLstSize(row); columnIdx++)
+        PackRead *read = pckReadNew(row);
+
+        for (unsigned int columnIdx = 0; columnIdx < 5; columnIdx++)
         {
-            if (varLstGet(row, columnIdx) == NULL)
+            if (pckReadNullP(read, .id = columnIdx + 1))
             {
                 THROW(
                     DbQueryError,
                     "unable to select some rows from pg_settings\n"
-                        "HINT: is the backup running as the postgres user?\n"
-                        "HINT: is the pg_read_all_settings role assigned for " PG_NAME " >= " PG_VERSION_10_STR "?");
+                    "HINT: is the backup running as the postgres user?\n"
+                    "HINT: is the pg_read_all_settings role assigned for " PG_NAME " >= " PG_VERSION_10_Z "?");
             }
         }
 
-        // Strip the minor version off since we don't need it.  In the future it might be a good idea to warn users when they are
+        // Restart the read to get the data
+        read = pckReadNew(row);
+
+        // Strip the minor version off since we don't need it. In the future it might be a good idea to warn users when they are
         // running an old minor version.
-        this->pub.pgVersion = varUIntForce(varLstGet(row, 0)) / 100 * 100;
+        this->pub.pgVersion = (unsigned int)pckReadI32P(read) / 100 * 100;
 
         // Store the data directory that PostgreSQL is running in, the archive mode, and archive command. These can be compared to
         // the configured pgBackRest directory, and archive settings checked for validity, when validating the configuration.
         // Also store the checkpoint timeout to warn in case a backup is requested without using the start-fast option.
         MEM_CONTEXT_BEGIN(this->pub.memContext)
         {
-            this->pub.pgDataPath = strDup(varStr(varLstGet(row, 1)));
-            this->pub.archiveMode = strDup(varStr(varLstGet(row, 2)));
-            this->pub.archiveCommand = strDup(varStr(varLstGet(row, 3)));
-            this->pub.checkpointTimeout = (TimeMSec)(varUIntForce(varLstGet(row, 4))) * MSEC_PER_SEC;
+            this->pub.pgDataPath = pckReadStrP(read);
+            this->pub.archiveMode = pckReadStrP(read);
+            this->pub.archiveCommand = pckReadStrP(read);
+            this->pub.checkpointTimeout = (TimeMSec)pckReadI32P(read) * MSEC_PER_SEC;
         }
         MEM_CONTEXT_END();
 
@@ -302,7 +319,7 @@ dbOpen(Db *this)
         this->pub.standby = dbIsInRecovery(this);
 
         // Get control file
-        this->pub.pgControl = pgControlFromFile(this->storage);
+        this->pub.pgControl = pgControlFromFile(this->storage, cfgOptionStrNull(cfgOptPgVersionForce));
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -324,29 +341,29 @@ dbBackupStartQuery(unsigned int pgVersion, bool startFast)
         strNew(),
         "select lsn::text as lsn,\n"
         "       pg_catalog.pg_%sfile_name(lsn)::text as wal_segment_name\n"
-        "  from pg_catalog.pg_start_backup('" PROJECT_NAME " backup started at ' || current_timestamp",
-        strZ(pgWalName(pgVersion)));
+        "  from pg_catalog.pg_%s('" PROJECT_NAME " backup started at ' || current_timestamp",
+        strZ(pgWalName(pgVersion)), pgVersion >= PG_VERSION_15 ? "backup_start" : "start_backup");
 
     // Start backup after immediate checkpoint
     if (startFast)
     {
-        strCatFmt(result, ", " TRUE_Z);
+        strCatZ(result, ", " TRUE_Z);
     }
     // Else start backup at the next scheduled checkpoint
     else
-        strCatFmt(result, ", " FALSE_Z);
+        strCatZ(result, ", " FALSE_Z);
 
     // Use non-exclusive backup mode when available
-    if (pgVersion >= PG_VERSION_96)
-        strCatFmt(result, ", " FALSE_Z);
+    if (pgVersion >= PG_VERSION_96 && pgVersion <= PG_VERSION_14)
+        strCatZ(result, ", " FALSE_Z);
 
     // Complete query
-    strCatFmt(result, ") as lsn");
+    strCatZ(result, ") as lsn");
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
 
-DbBackupStartResult
+FN_EXTERN DbBackupStartResult
 dbBackupStart(Db *const this, const bool startFast, const bool stopAuto, const bool archiveCheck)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -356,6 +373,8 @@ dbBackupStart(Db *const this, const bool startFast, const bool stopAuto, const b
         FUNCTION_LOG_PARAM(BOOL, archiveCheck);
     FUNCTION_LOG_END();
 
+    FUNCTION_AUDIT_STRUCT();
+
     ASSERT(this != NULL);
 
     DbBackupStartResult result = {.lsn = NULL};
@@ -363,8 +382,8 @@ dbBackupStart(Db *const this, const bool startFast, const bool stopAuto, const b
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Acquire the backup advisory lock to make sure that backups are not running from multiple backup servers against the same
-        // database cluster.  This lock helps make the stop-auto option safe.
-        if (!varBool(dbQueryColumn(this, STRDEF("select pg_catalog.pg_try_advisory_lock(" PG_BACKUP_ADVISORY_LOCK ")::bool"))))
+        // database cluster. This lock helps make the stop-auto option safe.
+        if (!pckReadBoolP(dbQueryColumn(this, STRDEF("select pg_catalog.pg_try_advisory_lock(" PG_BACKUP_ADVISORY_LOCK ")::bool"))))
         {
             THROW(
                 LockAcquireError,
@@ -375,12 +394,10 @@ dbBackupStart(Db *const this, const bool startFast, const bool stopAuto, const b
         // If stop-auto is enabled check for a running backup
         if (stopAuto)
         {
-            CHECK(AssertError, dbPgVersion(this) >= PG_VERSION_93, "feature not supported in PostgreSQL < " PG_VERSION_93_STR);
-
             // Feature is not needed for PostgreSQL >= 9.6 since backups are run in non-exclusive mode
             if (dbPgVersion(this) < PG_VERSION_96)
             {
-                if (varBool(dbQueryColumn(this, STRDEF("select pg_catalog.pg_is_in_backup()::bool"))))
+                if (pckReadBoolP(dbQueryColumn(this, STRDEF("select pg_catalog.pg_is_in_backup()::bool"))))
                 {
                     LOG_WARN(
                         "the cluster is already in backup mode but no " PROJECT_NAME " backup process is running."
@@ -397,7 +414,7 @@ dbBackupStart(Db *const this, const bool startFast, const bool stopAuto, const b
         {
             LOG_WARN_FMT(
                 CFGOPT_START_FAST " is disabled and " CFGOPT_DB_TIMEOUT " (%" PRIu64 "s) is smaller than the " PG_NAME
-                    " checkpoint_timeout (%" PRIu64 "s) - timeout may occur before the backup starts",
+                " checkpoint_timeout (%" PRIu64 "s) - timeout may occur before the backup starts",
                 dbDbTimeout(this) / MSEC_PER_SEC, dbCheckpointTimeout(this) / MSEC_PER_SEC);
         }
 
@@ -406,7 +423,7 @@ dbBackupStart(Db *const this, const bool startFast, const bool stopAuto, const b
 
         if (archiveCheck)
         {
-            walSegmentCheck = varStr(
+            walSegmentCheck = pckReadStrP(
                 dbQueryColumn(
                     this,
                     strNewFmt(
@@ -416,12 +433,12 @@ dbBackupStart(Db *const this, const bool startFast, const bool stopAuto, const b
         }
 
         // Start backup
-        VariantList *row = dbQueryRow(this, dbBackupStartQuery(dbPgVersion(this), startFast));
+        PackRead *const read = dbQueryRow(this, dbBackupStartQuery(dbPgVersion(this), startFast));
 
         // Make sure the backup start checkpoint was written to pg_control. This helps ensure that we have a consistent view of the
         // storage with PostgreSQL.
-        const PgControl pgControl = pgControlFromFile(this->storage);
-        const String *const lsnStart = varStr(varLstGet(row, 0));
+        const PgControl pgControl = pgControlFromFile(this->storage, cfgOptionStrNull(cfgOptPgVersionForce));
+        const String *const lsnStart = pckReadStrP(read);
 
         if (pgControl.checkpoint < pgLsnFromStr(lsnStart))
         {
@@ -430,20 +447,13 @@ dbBackupStart(Db *const this, const bool startFast, const bool stopAuto, const b
                 strZ(lsnStart));
         }
 
-        // If archive check then make sure WAL segment was switched on start backup
-        const String *const walSegmentName = varStr(varLstGet(row, 1));
+        // If archive check then make sure WAL segment was switched on start backup. The WAL segment might not have been switched if
+        // a switch occurred before the backup and nothing was written since then. In this case, force a switch so we can verify
+        // that archiving is functional before starting the backup.
+        const String *const walSegmentName = pckReadStrP(read);
 
         if (archiveCheck && strEq(walSegmentCheck, walSegmentName))
-        {
-            // If the version supports restore points then force a WAL switch
-            if (dbPgVersion(this) >= PG_VERSION_RESTORE_POINT)
-            {
-                dbWalSwitch(this);
-            }
-            // Else disable the check. All WAL will still be checked at the end of the backup.
-            else
-                walSegmentCheck = NULL;
-        }
+            dbWalSwitch(this);
 
         // Check that the WAL timeline matches what is in pg_control
         if (pgTimelineFromWalSegment(walSegmentName) != dbPgControl(this).timeline)
@@ -483,7 +493,7 @@ dbBackupStopQuery(unsigned int pgVersion)
         "       pg_catalog.pg_%sfile_name(lsn)::text as wal_segment_name",
         strZ(pgWalName(pgVersion)));
 
-    // For PostgreSQL >= 9.6 the backup label and tablespace map are returned from pg_stop_backup
+    // For PostgreSQL >= 9.6 the backup label and tablespace map are returned
     if (pgVersion >= PG_VERSION_96)
     {
         strCatZ(
@@ -494,35 +504,43 @@ dbBackupStopQuery(unsigned int pgVersion)
     }
 
     // Build stop backup function
-    strCatZ(
+    strCatFmt(
         result,
         "\n"
-        "  from pg_catalog.pg_stop_backup(");
+        "  from pg_catalog.pg_%s(",
+        pgVersion >= PG_VERSION_15 ? "backup_stop" : "stop_backup");
 
     // Use non-exclusive backup mode when available
-    if (pgVersion >= PG_VERSION_96)
-        strCatFmt(result, FALSE_Z);
+    if (pgVersion >= PG_VERSION_96 && pgVersion <= PG_VERSION_14)
+        strCatZ(result, FALSE_Z);
 
-    // Disable archive checking in pg_stop_backup() since we do this elsewhere
+    // Disable archive checking since we do this elsewhere
     if (pgVersion >= PG_VERSION_10)
-        strCatFmt(result, ", " FALSE_Z);
+    {
+        if (pgVersion <= PG_VERSION_14)
+            strCatZ(result, ", ");
+
+        strCatZ(result, FALSE_Z);
+    }
 
     // Complete query
-    strCatFmt(result, ")");
+    strCatZ(result, ")");
 
     if (pgVersion < PG_VERSION_96)
-        strCatFmt(result, " as lsn");
+        strCatZ(result, " as lsn");
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
 
-DbBackupStopResult
+FN_EXTERN DbBackupStopResult
 dbBackupStop(Db *this)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(DB, this);
     FUNCTION_LOG_END();
 
+    FUNCTION_AUDIT_STRUCT();
+
     ASSERT(this != NULL);
 
     DbBackupStopResult result = {.lsn = NULL};
@@ -530,24 +548,28 @@ dbBackupStop(Db *this)
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Stop backup
-        VariantList *row = dbQueryRow(this, dbBackupStopQuery(dbPgVersion(this)));
-
-        // Check if the tablespace map is empty
-        bool tablespaceMapEmpty =
-            dbPgVersion(this) >= PG_VERSION_96 ? strSize(strTrim(strDup(varStr(varLstGet(row, 3))))) == 0 : false;
+        PackRead *const read = dbQueryRow(this, dbBackupStopQuery(dbPgVersion(this)));
 
         // Return results
         MEM_CONTEXT_PRIOR_BEGIN()
         {
-            result.lsn = strDup(varStr(varLstGet(row, 0)));
-            result.walSegmentName = strDup(varStr(varLstGet(row, 1)));
+            result.lsn = pckReadStrP(read);
+            result.walSegmentName = pckReadStrP(read);
 
             if (dbPgVersion(this) >= PG_VERSION_96)
             {
-                result.backupLabel = strDup(varStr(varLstGet(row, 2)));
+                result.backupLabel = pckReadStrP(read);
 
-                if (!tablespaceMapEmpty)
-                    result.tablespaceMap = strDup(varStr(varLstGet(row, 3)));
+                // Return the tablespace map if it is not empty
+                String *const tablespaceMap = pckReadStrP(read);
+                String *const tablespaceMapTrim = strTrim(strDup(tablespaceMap));
+
+                if (!strEmpty(tablespaceMapTrim))
+                    result.tablespaceMap = tablespaceMap;
+                else
+                    strFree(tablespaceMap);
+
+                strFree(tablespaceMapTrim);
             }
         }
         MEM_CONTEXT_PRIOR_END();
@@ -558,7 +580,7 @@ dbBackupStop(Db *this)
 }
 
 /**********************************************************************************************************************************/
-VariantList *
+FN_EXTERN Pack *
 dbList(Db *this)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -566,11 +588,16 @@ dbList(Db *this)
     FUNCTION_LOG_END();
 
     FUNCTION_LOG_RETURN(
-        VARIANT_LIST, dbQuery(this, STRDEF("select oid::oid, datname::text, datlastsysoid::oid from pg_catalog.pg_database")));
+        PACK,
+        dbQuery(
+            this, pgClientQueryResultAny,
+            STRDEF(
+                "select oid::oid, datname::text, (select oid::oid from pg_catalog.pg_database where datname = 'template0')"
+                " as datlastsysoid from pg_catalog.pg_database")));
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 dbReplayWait(Db *const this, const String *const targetLsn, const uint32_t targetTimeline, const TimeMSec timeout)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -587,10 +614,6 @@ dbReplayWait(Db *const this, const String *const targetLsn, const uint32_t targe
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        // Check that the timeline matches the primary
-        if (dbPgControl(this).timeline != targetTimeline)
-            THROW_FMT(DbMismatchError, "standby is on timeline %u but expected %u", dbPgControl(this).timeline, targetTimeline);
-
         // Standby checkpoint before the backup started must be <= the target LSN. If not, it indicates that the standby was ahead
         // of the primary and cannot be following it.
         if (dbPgControl(this).checkpoint > pgLsnFromStr(targetLsn))
@@ -633,10 +656,10 @@ dbReplayWait(Db *const this, const String *const targetLsn, const uint32_t targe
                 strZ(replayLsnFunction));
 
             // Execute the query and get replayLsn
-            VariantList *row = dbQueryRow(this, query);
-            replayLsn = varStr(varLstGet(row, 0));
+            PackRead *read = dbQueryRow(this, query);
+            replayLsn = pckReadStrP(read);
 
-            // Error when replayLsn is null which indicates that this is not a standby.  This should have been sorted out before we
+            // Error when replayLsn is null which indicates that this is not a standby. This should have been sorted out before we
             // connected but it's possible that the standby was promoted in the meantime.
             if (replayLsn == NULL)
             {
@@ -647,10 +670,10 @@ dbReplayWait(Db *const this, const String *const targetLsn, const uint32_t targe
                     strZ(replayLsnFunction));
             }
 
-            targetReached = varBool(varLstGet(row, 1));
+            targetReached = pckReadBoolP(read);
 
             // If the target has not been reached but progress is being made then reset the timer
-            if (!targetReached && varLstSize(row) > 2 && varBool(varLstGet(row, 2)))
+            if (!targetReached && pckReadBoolP(read, .defaultValue = true))
                 wait = waitNew(timeout);
 
             protocolKeepAlive();
@@ -688,9 +711,9 @@ dbReplayWait(Db *const this, const String *const targetLsn, const uint32_t targe
                     lsnName, strZ(targetLsn), lsnName);
 
                 // Execute the query and get checkpointLsn
-                VariantList *row = dbQueryRow(this, query);
-                targetReached = varBool(varLstGet(row, 0));
-                checkpointLsn = varStr(varLstGet(row, 1));
+                PackRead *const read = dbQueryRow(this, query);
+                targetReached = pckReadBoolP(read);
+                checkpointLsn = pckReadStrP(read);
 
                 protocolKeepAlive();
             }
@@ -704,6 +727,13 @@ dbReplayWait(Db *const this, const String *const targetLsn, const uint32_t targe
                     strZ(checkpointLsn));
             }
         }
+
+        // Reload pg_control in case timeline was updated by the checkpoint
+        this->pub.pgControl = pgControlFromFile(this->storage, cfgOptionStrNull(cfgOptPgVersionForce));
+
+        // Check that the timeline matches the primary
+        if (dbPgControl(this).timeline != targetTimeline)
+            THROW_FMT(DbMismatchError, "standby is on timeline %u but expected %u", dbPgControl(this).timeline, targetTimeline);
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -711,7 +741,7 @@ dbReplayWait(Db *const this, const String *const targetLsn, const uint32_t targe
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 dbPing(Db *const this, const bool force)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -753,30 +783,40 @@ dbPing(Db *const this, const bool force)
 }
 
 /**********************************************************************************************************************************/
-VariantList *
+FN_EXTERN Pack *
 dbTablespaceList(Db *this)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(DB, this);
     FUNCTION_LOG_END();
 
-    FUNCTION_LOG_RETURN(VARIANT_LIST, dbQuery(this, STRDEF("select oid::oid, spcname::text from pg_catalog.pg_tablespace")));
+    FUNCTION_LOG_RETURN(
+        PACK,
+        dbQuery(this, pgClientQueryResultAny, STRDEF("select oid::oid, spcname::text from pg_catalog.pg_tablespace")));
 }
 
 /**********************************************************************************************************************************/
-TimeMSec
+FN_EXTERN TimeMSec
 dbTimeMSec(Db *this)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(DB, this);
     FUNCTION_LOG_END();
 
-    FUNCTION_LOG_RETURN(
-        TIME_MSEC, varUInt64Force(dbQueryColumn(this, STRDEF("select (extract(epoch from clock_timestamp()) * 1000)::bigint"))));
+    TimeMSec result;
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        result = (TimeMSec)pckReadI64P(
+            dbQueryColumn(this, STRDEF("select (extract(epoch from clock_timestamp()) * 1000)::bigint")));
+    }
+    MEM_CONTEXT_TEMP_END();
+
+    FUNCTION_LOG_RETURN(TIME_MSEC, result);
 }
 
 /**********************************************************************************************************************************/
-String *
+FN_EXTERN String *
 dbWalSwitch(Db *this)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -789,14 +829,12 @@ dbWalSwitch(Db *this)
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        // Create a restore point to ensure current WAL will be archived.  For versions < 9.1 activity will need to be generated by
-        // the user if there have been no writes since the last WAL switch.
-        if (dbPgVersion(this) >= PG_VERSION_RESTORE_POINT)
-            dbQueryColumn(this, STRDEF("select pg_catalog.pg_create_restore_point('" PROJECT_NAME " Archive Check')::text"));
+        // Create a restore point to ensure current WAL will be archived
+        dbQueryColumn(this, STRDEF("select pg_catalog.pg_create_restore_point('" PROJECT_NAME " Archive Check')::text"));
 
         // Request a WAL segment switch
         const char *walName = strZ(pgWalName(dbPgVersion(this)));
-        const String *walFileName = varStr(
+        const String *walFileName = pckReadStrP(
             dbQueryColumn(this, strNewFmt("select pg_catalog.pg_%sfile_name(pg_catalog.pg_switch_%s())::text", walName, walName)));
 
         // Copy WAL segment name to the prior context
@@ -812,10 +850,17 @@ dbWalSwitch(Db *this)
 }
 
 /**********************************************************************************************************************************/
-String *
-dbToLog(const Db *this)
+FN_EXTERN void
+dbToLog(const Db *const this, StringStatic *const debugLog)
 {
-    return strNewFmt(
-        "{client: %s, remoteClient: %s}", this->client == NULL ? NULL_Z : strZ(pgClientToLog(this->client)),
-        this->remoteClient == NULL ? NULL_Z : strZ(protocolClientToLog(this->remoteClient)));
+    strStcCat(debugLog, "{client: ");
+    strStcResultSizeInc(
+        debugLog, FUNCTION_LOG_OBJECT_FORMAT(this->client, pgClientToLog, strStcRemains(debugLog), strStcRemainsSize(debugLog)));
+
+    strStcCat(debugLog, ", remoteClient: ");
+    strStcResultSizeInc(
+        debugLog,
+        FUNCTION_LOG_OBJECT_FORMAT(
+            this->remoteClient, protocolClientToLog, strStcRemains(debugLog), strStcRemainsSize(debugLog)));
+    strStcCatChr(debugLog, '}');
 }
