@@ -9,7 +9,9 @@ PostgreSQL Interface
 #include "common/log.h"
 #include "common/memContext.h"
 #include "common/regExp.h"
+#include "common/time.h"
 #include "postgres/interface.h"
+#include "postgres/interface/crc32.h"
 #include "postgres/interface/static.vendor.h"
 #include "postgres/version.h"
 #include "storage/helper.h"
@@ -44,8 +46,10 @@ STRING_STATIC(PG_NAME_LOCATION_STR,                                 "location");
 
 /***********************************************************************************************************************************
 The control file is 8192 bytes but only the first 512 bytes are used to prevent torn pages even on really old storage with 512-byte
-sectors. This is true across all versions of PostgreSQL.
+sectors. This is true across all versions of PostgreSQL. Unfortunately, this is not sufficient to prevent torn pages during
+concurrent reads and writes so retries are required.
 ***********************************************************************************************************************************/
+#define PG_CONTROL_SIZE                                             8192
 #define PG_CONTROL_DATA_SIZE                                        512
 
 /***********************************************************************************************************************************
@@ -64,6 +68,9 @@ typedef struct PgInterface
     // Convert pg_control to a common data structure
     PgControl (*control)(const unsigned char *);
 
+    // Get control crc offset
+    size_t (*controlCrcOffset)(void);
+
     // Get the control version for this version of PostgreSQL
     uint32_t (*controlVersion)(void);
 
@@ -222,6 +229,47 @@ pgControlFromBuffer(const Buffer *controlFile, const String *const pgVersionForc
     PgControl result = interface->control(bufPtrConst(controlFile));
     result.version = interface->version;
 
+    // Check CRC
+    size_t crcOffset = interface->controlCrcOffset();
+
+    do
+    {
+        // Calculate CRC and retrieve expected CRC
+        const uint32_t crcCalculated =
+            result.version > PG_VERSION_94 ?
+                crc32cOne(bufPtrConst(controlFile), crcOffset) : crc32One(bufPtrConst(controlFile), crcOffset);
+        const uint32_t crcExpected = *((uint32_t *)(bufPtrConst(controlFile) + crcOffset));
+
+        // If CRC does not match
+        if (crcCalculated != crcExpected)
+        {
+            // If version is forced then the CRC might be later in the file (assuming the fork added extra fields to pg_control).
+            // Increment the offset by CRC data size and continue to try again.
+            if (pgVersionForce != NULL)
+            {
+                crcOffset += sizeof(uint32_t);
+
+                if (crcOffset <= bufUsed(controlFile) - sizeof(uint32_t))
+                    continue;
+            }
+
+            // If no retry then error
+            THROW_FMT(
+                ChecksumError,
+                "calculated " PG_FILE_PGCONTROL " checksum does not match expected value\n"
+                "HINT: calculated 0x%x but expected value is 0x%x\n"
+                "%s"
+                "HINT: is " PG_FILE_PGCONTROL " corrupt?\n"
+                "HINT: does " PG_FILE_PGCONTROL " have a different layout than expected?",
+                crcCalculated, crcExpected,
+                pgVersionForce == NULL ? "" : "HINT: checksum values may be misleading due to forced version scan\n");
+        }
+
+        // Do not retry if the CRC is valid
+        break;
+    }
+    while (true);
+
     // Check the segment size
     pgWalSegmentSizeCheck(result.version, result.walSegmentSize);
 
@@ -232,8 +280,20 @@ pgControlFromBuffer(const Buffer *controlFile, const String *const pgVersionForc
     FUNCTION_LOG_RETURN(PG_CONTROL, result);
 }
 
-FN_EXTERN PgControl
-pgControlFromFile(const Storage *storage, const String *const pgVersionForce)
+// Helper to compare control data to last read
+static bool
+pgControlBufferEq(const Buffer *const last, const Buffer *const current)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(BUFFER, last);
+        FUNCTION_TEST_PARAM(BUFFER, current);
+    FUNCTION_TEST_END();
+
+    FUNCTION_TEST_RETURN(BOOL, last != NULL && bufEq(last, current));
+}
+
+FN_EXTERN Buffer *
+pgControlBufferFromFile(const Storage *const storage, const String *const pgVersionForce)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE, storage);
@@ -242,18 +302,75 @@ pgControlFromFile(const Storage *storage, const String *const pgVersionForce)
 
     ASSERT(storage != NULL);
 
-    PgControl result = {0};
+    Buffer *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        // Read control file
-        Buffer *controlFile = storageGetP(
-            storageNewReadP(storage, STRDEF(PG_PATH_GLOBAL "/" PG_FILE_PGCONTROL)), .exactSize = PG_CONTROL_DATA_SIZE);
+        // On filesystems that do not implement atomicity of concurrent reads and writes, we might get garbage if the server is
+        // writing the pg_control file at the same time as we try to read it. Keep trying until success or we read the same data
+        // twice in a row. Do not use a timeout here because there are plenty of other errors that can happen and we don't want to
+        // wait for them.
+        Buffer *controlFileLast = NULL;
+        bool done = false;
 
-        result = pgControlFromBuffer(controlFile, pgVersionForce);
+        do
+        {
+            // Read control file
+            Buffer *const controlFile = storageGetP(
+                storageNewReadP(storage, STRDEF(PG_PATH_GLOBAL "/" PG_FILE_PGCONTROL)), .exactSize = PG_CONTROL_DATA_SIZE);
+
+            TRY_BEGIN()
+            {
+                // Check that control data is valid
+                pgControlFromBuffer(controlFile, pgVersionForce);
+
+                // Create a buffer of the correct size to hold pg_control and zero out the remaining portion
+                result = bufNew(PG_CONTROL_SIZE);
+
+                bufCat(result, controlFile);
+                memset(bufPtr(result) + bufUsed(controlFile), 0, bufSize(result) - bufUsed(controlFile));
+                bufUsedSet(result, bufSize(result));
+                bufMove(result, memContextPrior());
+
+                done = true;
+            }
+            CATCH_ANY()
+            {
+                // If we get the same bad data twice in a row then error
+                if (pgControlBufferEq(controlFileLast, controlFile))
+                    RETHROW();
+
+                // Copy data to last
+                bufFree(controlFileLast);
+                controlFileLast = controlFile;
+
+                // Sleep to let data stabilize
+                sleepMSec(50);
+            }
+            TRY_END();
+        }
+        while (!done);
     }
     MEM_CONTEXT_TEMP_END();
 
+    FUNCTION_LOG_RETURN(BUFFER, result);
+}
+
+FN_EXTERN PgControl
+pgControlFromFile(const Storage *const storage, const String *const pgVersionForce)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(STORAGE, storage);
+        FUNCTION_LOG_PARAM(STRING, pgVersionForce);
+    FUNCTION_LOG_END();
+
+    ASSERT(storage != NULL);
+
+    Buffer *const buffer = pgControlBufferFromFile(storage, pgVersionForce);
+    PgControl result = pgControlFromBuffer(buffer, pgVersionForce);
+
+    bufFree(buffer);
+
     FUNCTION_LOG_RETURN(PG_CONTROL, result);
 }
 
