@@ -64,7 +64,6 @@ sub new
             strImage => containerRepo() . ':' . testRunGet()->vm() . "-test",
             bTls => $oParam->{bTls},
             strBackupDestination => $$oParam{strBackupDestination},
-            oLogTest => $$oParam{oLogTest},
             bStandby => $$oParam{bStandby},
             bRepoLocal => $oParam->{bRepoLocal},
             bRepoEncrypt => $oParam->{bRepoEncrypt},
@@ -100,9 +99,13 @@ sub new
 
             if (!defined($$oParam{bStandby}) || !$$oParam{bStandby})
             {
-                &log(WARN, "Testing against ${strDbVersionActual} ${strDevVersion} version");
+                &log(WARN, 'Testing against ' . trim($strOutLog) . " ${strDevVersion}");
             }
         }
+        elsif (!defined($$oParam{bStandby}) || !$$oParam{bStandby})
+        {
+            &log(INFO, 'Testing against ' . trim($strOutLog));
+        }
 
         # Don't run unit tests for unsupported versions
         my @stryVersionSupport = versionSupport();
@@ -347,15 +350,13 @@ sub clusterCreate
     # Set defaults
     my $strWalPath = defined($$hParam{strWalPath}) ? $$hParam{strWalPath} : $self->dbPath() . '/pg_' . $self->walId();
 
-    # Don't link WAL directory for versions < 9.2 because some recovery scenarios won't work
     $self->executeSimple(
         $self->pgBinPath() . '/initdb ' .
         ($self->pgVersion() >= PG_VERSION_93 ? ' -k' : '') .
-        ($self->pgVersion() >= PG_VERSION_92 ? ' --' . $self->walId() . "dir=${strWalPath}" : '') .
         ($self->pgVersion() >= PG_VERSION_11 ? ' --wal-segsize=1' : '') .
-        ' --pgdata=' . $self->dbBasePath() . ' --auth=trust');
+        ' --' . $self->walId() . "dir=${strWalPath}" . ' --pgdata=' . $self->dbBasePath() . ' --auth=trust');
 
-    if (!$self->standby() && $self->pgVersion() >= PG_VERSION_HOT_STANDBY)
+    if (!$self->standby())
     {
         $self->executeSimple(
             "echo 'host replication replicator db-standby trust' >> " . $self->dbBasePath() . '/pg_hba.conf');
@@ -365,7 +366,7 @@ sub clusterCreate
         {bHotStandby => $$hParam{bHotStandby}, bArchive => $$hParam{bArchive}, bArchiveAlways => $$hParam{bArchiveAlways},
          bArchiveInvalid => $$hParam{bArchiveInvalid}});
 
-    if (!$self->standby() && $self->pgVersion() >= PG_VERSION_HOT_STANDBY)
+    if (!$self->standby())
     {
         $self->sqlExecute("create user replicator replication", {bCommit =>true});
     }
@@ -435,11 +436,20 @@ sub clusterStart
     # works since pg_stop_backup() is marked parallel safe and will error if run in a worker.
     if ($self->pgVersion() >= PG_VERSION_96)
     {
-         $strCommand .= " -c force_parallel_mode='on' -c max_parallel_workers_per_gather=2";
+        if ($self->pgVersion() >= PG_VERSION_16)
+        {
+            $strCommand .= " -c debug_parallel_query='on'";
+        }
+        else
+        {
+            $strCommand .= " -c force_parallel_mode='on'";
+        }
+
+         $strCommand .= " -c max_parallel_workers_per_gather=2";
     }
 
     $strCommand .=
-        ($self->pgVersion() >= PG_VERSION_HOT_STANDBY ? ' -c max_wal_senders=3' : '') .
+        ' -c max_wal_senders=3' .
         ' -c listen_addresses=\'*\'' .
         ' -c log_directory=\'' . $self->pgLogPath() . "'" .
         ' -c log_filename=\'' . basename($self->pgLogFile()) . "'" .
@@ -467,6 +477,7 @@ sub clusterStop
 
     # Set defaults
     my $bIgnoreLogError = defined($$hParam{bIgnoreLogError}) ? $$hParam{bIgnoreLogError} : false;
+    my $bStop = defined($hParam->{bStop}) ? $$hParam{bStop} : true;
 
     # Disconnect user session
     $self->sqlDisconnect();
@@ -481,7 +492,7 @@ sub clusterStop
     }
 
     # If pg process is running then stop the cluster
-    if (-e $self->dbBasePath() . '/' . DB_FILE_POSTMTRPID)
+    if ($bStop && -e $self->dbBasePath() . '/' . DB_FILE_POSTMTRPID)
     {
         $self->executeSimple($self->pgBinPath() . '/pg_ctl stop -D ' . $self->dbBasePath() . ' -w -s -m fast');
     }
