@@ -52,50 +52,47 @@ sub run
 
     foreach my $rhRun
     (
-        {pg => '9.0', repoDest => HOST_DB_PRIMARY, tls => 0, storage =>   GCS, encrypt => 1, compress =>  BZ2, repo => 2, bnd => 1},
-        {pg => '9.1', repoDest => HOST_DB_STANDBY, tls => 1, storage =>   GCS, encrypt => 0, compress =>   GZ, repo => 1, bnd => 0},
-        {pg => '9.2', repoDest => HOST_DB_STANDBY, tls => 0, storage => POSIX, encrypt => 1, compress => NONE, repo => 1, bnd => 1},
-        {pg => '9.3', repoDest =>     HOST_BACKUP, tls => 0, storage => AZURE, encrypt => 0, compress => NONE, repo => 2, bnd => 0},
-        {pg => '9.4', repoDest => HOST_DB_STANDBY, tls => 0, storage => POSIX, encrypt => 1, compress =>  LZ4, repo => 1, bnd => 1},
-        {pg => '9.5', repoDest =>     HOST_BACKUP, tls => 1, storage =>    S3, encrypt => 0, compress =>  BZ2, repo => 1, bnd => 0},
-        {pg => '9.6', repoDest =>     HOST_BACKUP, tls => 0, storage => POSIX, encrypt => 0, compress => NONE, repo => 2, bnd => 1},
-        {pg =>  '10', repoDest => HOST_DB_STANDBY, tls => 1, storage =>    S3, encrypt => 1, compress =>   GZ, repo => 2, bnd => 0},
-        {pg =>  '11', repoDest =>     HOST_BACKUP, tls => 1, storage => AZURE, encrypt => 0, compress =>  ZST, repo => 2, bnd => 1},
-        {pg =>  '12', repoDest =>     HOST_BACKUP, tls => 0, storage =>    S3, encrypt => 1, compress =>  LZ4, repo => 1, bnd => 0},
-        {pg =>  '13', repoDest => HOST_DB_STANDBY, tls => 1, storage =>   GCS, encrypt => 0, compress =>  ZST, repo => 1, bnd => 1},
-        {pg =>  '14', repoDest =>     HOST_BACKUP, tls => 0, storage => POSIX, encrypt => 1, compress =>  LZ4, repo => 2, bnd => 0},
+        {pg => '9.3', dst =>     'backup', tls => 0, stg => AZURE, enc => 0, cmp => NONE, rt => 2, bnd => 0, bi => 0},
+        {pg => '9.4', dst => 'db-standby', tls => 0, stg => POSIX, enc => 1, cmp =>  LZ4, rt => 1, bnd => 1, bi => 0},
+        {pg => '9.5', dst =>     'backup', tls => 1, stg =>   GCS, enc => 0, cmp =>  BZ2, rt => 1, bnd => 0, bi => 1},
+        {pg => '9.6', dst =>     'backup', tls => 0, stg => POSIX, enc => 0, cmp => NONE, rt => 2, bnd => 1, bi => 1},
+        {pg =>  '10', dst =>  'sftp-srvr', tls => 0, stg =>  SFTP, enc => 1, cmp =>   GZ, rt => 1, bnd => 1, bi => 0},
+        {pg =>  '11', dst =>     'backup', tls => 1, stg => AZURE, enc => 0, cmp =>  ZST, rt => 2, bnd => 0, bi => 0},
+        {pg =>  '12', dst =>     'backup', tls => 0, stg =>    S3, enc => 1, cmp =>  LZ4, rt => 1, bnd => 0, bi => 1},
+        {pg =>  '13', dst => 'db-standby', tls => 1, stg =>   GCS, enc => 0, cmp =>  ZST, rt => 1, bnd => 1, bi => 1},
+        {pg =>  '14', dst =>  'sftp-srvr', tls => 0, stg =>  SFTP, enc => 0, cmp =>  LZ4, rt => 1, bnd => 1, bi => 0},
+        {pg =>  '15', dst => 'db-standby', tls => 0, stg => AZURE, enc => 0, cmp => NONE, rt => 2, bnd => 1, bi => 1},
+        {pg =>  '16', dst => 'db-standby', tls => 0, stg =>    S3, enc => 1, cmp => NONE, rt => 1, bnd => 0, bi => 0},
     )
     {
         # Only run tests for this pg version
         next if ($rhRun->{pg} ne $self->pgVersion());
 
         # Get run parameters
-        my $bHostBackup = $rhRun->{repoDest} eq HOST_BACKUP ? true : false;
-        my $bHostStandby = $self->pgVersion() >= PG_VERSION_HOT_STANDBY ? true : false;
+        my $bHostBackup = $rhRun->{dst} eq HOST_BACKUP ? true : false;
         my $bTls = $rhRun->{tls};
-        my $strBackupDestination = $rhRun->{repoDest};
-        my $strStorage = $rhRun->{storage};
-        my $bRepoEncrypt = $rhRun->{encrypt};
-        my $strCompressType = $rhRun->{compress};
-        my $iRepoTotal = $rhRun->{repo};
+        my $strBackupDestination = $rhRun->{dst};
+        my $strStorage = $rhRun->{stg};
+        my $bRepoEncrypt = $rhRun->{enc};
+        my $strCompressType = $rhRun->{cmp};
+        my $iRepoTotal = $rhRun->{rt};
         my $bBundle = $rhRun->{bnd};
+        my $bBlockIncr = $rhRun->{bi};
 
-        # Use a specific VM and version of PostgreSQL for expect testing. This version will also be used to run tests that are not
-        # version specific.
-        my $bExpectVersion = $self->vm() eq VM_EXPECT && $self->pgVersion() eq PG_VERSION_96;
+        # Some tests are not version specific so only run them on a single version of PostgreSQL
+        my $bNonVersionSpecific = $self->pgVersion() eq PG_VERSION_96;
 
         # Increment the run, log, and decide whether this unit test should be run
-        next if (!$self->begin(
-            "bkp ${bHostBackup}, sby ${bHostStandby}, tls ${bTls}, dst ${strBackupDestination}, cmp ${strCompressType}" .
-                ", storage ${strStorage}, enc ${bRepoEncrypt}",
-            $bExpectVersion));
+        next if !$self->begin(
+            "bkp ${bHostBackup}, tls ${bTls}, dst ${strBackupDestination}, cmp ${strCompressType}, storage ${strStorage}" .
+                ", enc ${bRepoEncrypt}, bi ${bBlockIncr}");
 
         # Create hosts, file object, and config
         my ($oHostDbPrimary, $oHostDbStandby, $oHostBackup) = $self->setup(
-            false, $self->expect(),
-            {bHostBackup => $bHostBackup, bStandby => $bHostStandby, bTls => $bTls, strBackupDestination => $strBackupDestination,
-             strCompressType => $strCompressType, bArchiveAsync => false, strStorage => $strStorage,
-             bRepoEncrypt => $bRepoEncrypt, iRepoTotal => $iRepoTotal, bBundle => $bBundle});
+            false,
+            {bHostBackup => $bHostBackup, bStandby => true, bTls => $bTls, strBackupDestination => $strBackupDestination,
+                strCompressType => $strCompressType, bArchiveAsync => false, strStorage => $strStorage,
+                bRepoEncrypt => $bRepoEncrypt, iRepoTotal => $iRepoTotal, bBundle => $bBundle, bBlockIncr => $bBlockIncr});
 
         # Some commands will fail because of the bogus host created when a standby is present. These options reset the bogus host
         # so it won't interfere with commands that won't tolerate a connection failure.
@@ -104,7 +101,7 @@ sub run
                 ' --reset-pg2-path' :
             '';
 
-        # If S3 set process max to 2.  This seems like the best place for parallel testing since it will help speed S3 processing
+        # If S3 set process max to 2. This seems like the best place for parallel testing since it will help speed S3 processing
         # without slowing down the other tests too much.
         if ($strStorage eq S3)
         {
@@ -158,12 +155,12 @@ sub run
         # --------------------------------------------------------------------------------------------------------------------------
         my $strComment = 'verify check command runs successfully';
 
-        $oHostDbPrimary->check($strComment, {iTimeout => 5});
+        $oHostDbPrimary->check($strComment, {iTimeout => 10, bStanza => false});
 
         # Also run check on the backup host when present
         if ($bHostBackup)
         {
-            $oHostBackup->check($strComment, {iTimeout => 5, strOptionalParam => $strBogusReset});
+            $oHostBackup->check($strComment, {iTimeout => 10, strOptionalParam => $strBogusReset});
         }
 
         # Restart the cluster ignoring any errors in the postgresql log
@@ -208,9 +205,9 @@ sub run
         }
 
         # Make a new backup with expire-auto disabled then run the expire command and compare backup numbers to ensure that expire
-        # was really disabled. This test is not version specific so is run on only the expect version.
+        # was really disabled. This test is not version specific so is run on only one version.
         #---------------------------------------------------------------------------------------------------------------------------
-        if ($bExpectVersion)
+        if ($bNonVersionSpecific)
         {
             my $oBackupInfo = new pgBackRestTest::Env::BackupInfo($oHostBackup->repoBackupPath());
             push(my @backupLst1, $oBackupInfo->list());
@@ -236,8 +233,8 @@ sub run
         # Enabled async archiving
         $oHostBackup->configUpdate({&CFGDEF_SECTION_GLOBAL => {'archive-async' => 'y'}});
 
-        # Kick out a bunch of archive logs to exercise async archiving.  Only do this when compressed and remote to slow it
-        # down enough to make it evident that the async process is working.
+        # Kick out a bunch of archive logs to exercise async archiving. Only do this when compressed and remote to slow it down
+        # enough to make it evident that the async process is working.
         if ($strCompressType ne NONE && $strBackupDestination eq HOST_BACKUP)
         {
             &log(INFO, '    multiple wal switches to exercise async archiving');
@@ -255,102 +252,91 @@ sub run
 
         # Setup replica
         #---------------------------------------------------------------------------------------------------------------------------
-        if ($bHostStandby)
+        my %oRemapHash;
+        $oRemapHash{&MANIFEST_TARGET_PGDATA} = $oHostDbStandby->dbBasePath();
+
+        $oHostDbStandby->linkRemap($oManifest->walPath(), $oHostDbStandby->dbPath() . '/' . $oManifest->walPath());
+
+        $oHostDbStandby->restore(
+            'restore backup on replica', 'latest',
+            {rhRemapHash => \%oRemapHash, strType => CFGOPTVAL_RESTORE_TYPE_STANDBY,
+                strOptionalParam =>
+                    ' --recovery-option="primary_conninfo=host=' . HOST_DB_PRIMARY .
+                    ' port=' . $oHostDbPrimary->pgPort() . ' user=replicator"'});
+
+        $oHostDbStandby->clusterStart({bHotStandby => true});
+
+        # Make sure streaming replication is on
+        $oHostDbPrimary->sqlSelectOneTest(
+            "select client_addr || '-' || state from pg_stat_replication", $oHostDbStandby->ipGet() . '/32-streaming');
+
+        # Check that the cluster was restored properly
+        $oHostDbStandby->sqlSelectOneTest('select message from test', $strFullMessage);
+
+        # Update message for standby
+        $oHostDbPrimary->sqlExecute("update test set message = '$strStandbyMessage'");
+
+        if (!$bTls)
         {
-            my %oRemapHash;
-            $oRemapHash{&MANIFEST_TARGET_PGDATA} = $oHostDbStandby->dbBasePath();
-
-            if ($oHostDbStandby->pgVersion() >= PG_VERSION_92)
+            # If there is only a primary and a replica and the replica is the backup destination, then if pg2-host and
+            # pg256-host are BOGUS, confirm failure to reach the primary
+            if (!$bHostBackup && $strBackupDestination eq HOST_DB_STANDBY)
             {
-                $oHostDbStandby->linkRemap($oManifest->walPath(), $oHostDbStandby->dbPath() . '/' . $oManifest->walPath());
+                my $strStandbyBackup = $oHostBackup->backup(
+                    CFGOPTVAL_BACKUP_TYPE_FULL, 'backup from standby, failure to reach primary',
+                    {bStandby => true, iExpectedExitStatus => ERROR_DB_CONNECT, strOptionalParam => '--pg256-host=' . BOGUS});
             }
-
-            $oHostDbStandby->restore(
-                'restore backup on replica', 'latest',
-                {rhRemapHash => \%oRemapHash, strType => CFGOPTVAL_RESTORE_TYPE_STANDBY,
-                    strOptionalParam =>
-                        ' --recovery-option="primary_conninfo=host=' . HOST_DB_PRIMARY .
-                        ' port=' . $oHostDbPrimary->pgPort() . ' user=replicator"'});
-
-            $oHostDbStandby->clusterStart({bHotStandby => true});
-
-            # Make sure streaming replication is on
-            $oHostDbPrimary->sqlSelectOneTest(
-                "select client_addr || '-' || state from pg_stat_replication", $oHostDbStandby->ipGet() . '/32-streaming');
-
-            # Check that the cluster was restored properly
-            $oHostDbStandby->sqlSelectOneTest('select message from test', $strFullMessage);
-
-            # Update message for standby
-            $oHostDbPrimary->sqlExecute("update test set message = '$strStandbyMessage'");
-
-            if ($oHostDbStandby->pgVersion() >= PG_VERSION_BACKUP_STANDBY && !$bTls)
+            else
             {
-                # If there is only a primary and a replica and the replica is the backup destination, then if pg2-host and
-                # pg256-host are BOGUS, confirm failure to reach the primary
-                if (!$bHostBackup && $bHostStandby && $strBackupDestination eq HOST_DB_STANDBY)
-                {
-                    my $strStandbyBackup = $oHostBackup->backup(
-                        CFGOPTVAL_BACKUP_TYPE_FULL, 'backup from standby, failure to reach primary',
-                        {bStandby => true, iExpectedExitStatus => ERROR_DB_CONNECT, strOptionalParam => '--pg256-host=' . BOGUS});
-                }
-                else
-                {
-                    my $strStandbyBackup = $oHostBackup->backup(
-                        CFGOPTVAL_BACKUP_TYPE_FULL, 'backup from standby, failure to access at least one standby',
-                        {bStandby => true, iExpectedExitStatus => ERROR_DB_CONNECT, strOptionalParam => '--pg256-host=' . BOGUS});
-                }
+                my $strStandbyBackup = $oHostBackup->backup(
+                    CFGOPTVAL_BACKUP_TYPE_FULL, 'backup from standby, failure to access at least one standby',
+                    {bStandby => true, iExpectedExitStatus => ERROR_DB_CONNECT, strOptionalParam => '--pg256-host=' . BOGUS});
             }
-
-            my $strStandbyBackup = $oHostBackup->backup(
-                CFGOPTVAL_BACKUP_TYPE_FULL, 'backup from standby',
-                {bStandby => true,
-                 iExpectedExitStatus => $oHostDbStandby->pgVersion() >= PG_VERSION_BACKUP_STANDBY ? undef : ERROR_CONFIG,
-                 strOptionalParam => '--repo1-retention-full=1'});
-
-            if ($oHostDbStandby->pgVersion() >= PG_VERSION_BACKUP_STANDBY)
-            {
-                $strFullBackup = $strStandbyBackup;
-            }
-
-            # ??? Removed temporarily until manifest build can be brought back into the check command
-            # # Create a directory in pg_data location that is only readable by root to ensure manifest->build is called by check
-            # my $strDir = $oHostDbStandby->dbBasePath() . '/rootreaddir';
-            # executeTest('sudo mkdir ' . $strDir);
-            # executeTest("sudo chown root:root ${strDir}");
-            # executeTest("sudo chmod 400 ${strDir}");
-            #
-            # my $strComment = 'confirm standby manifest->build executed';
-            #
-            # # If there is an invalid host, the final error returned from check will be the inability to resolve the name which is
-            # # an open error instead of a read error
-            # if (!$oHostDbStandby->bogusHost())
-            # {
-            #     $oHostDbStandby->check($strComment, {iTimeout => 5, iExpectedExitStatus => ERROR_PATH_OPEN});
-            # }
-            # else
-            # {
-            #     $oHostDbStandby->check($strComment, {iTimeout => 5, iExpectedExitStatus => ERROR_FILE_READ});
-            # }
-            #
-            # # Remove the directory in pg_data location that is only readable by root
-            # executeTest("sudo rmdir ${strDir}");
-
-            # Confirm the check command runs without error on a standby (when a bogus host is not configured)
-            $oHostDbStandby->check('verify check command on standby', {strOptionalParam => $strBogusReset});
-
-            # Shutdown the standby before creating tablespaces (this will error since paths are different)
-            $oHostDbStandby->clusterStop({bIgnoreLogError => true});
         }
 
+        my $strStandbyBackup = $oHostBackup->backup(
+            CFGOPTVAL_BACKUP_TYPE_FULL, 'backup from standby',
+            {bStandby => true, iExpectedExitStatus => undef, strOptionalParam => '--repo1-retention-full=1'});
+
+        $strFullBackup = $strStandbyBackup;
+
+        # ??? Removed temporarily until manifest build can be brought back into the check command
+        # # Create a directory in pg_data location that is only readable by root to ensure manifest->build is called by check
+        # my $strDir = $oHostDbStandby->dbBasePath() . '/rootreaddir';
+        # executeTest('sudo mkdir ' . $strDir);
+        # executeTest("sudo chown root:root ${strDir}");
+        # executeTest("sudo chmod 400 ${strDir}");
+        #
+        # my $strComment = 'confirm standby manifest->build executed';
+        #
+        # # If there is an invalid host, the final error returned from check will be the inability to resolve the name which is
+        # # an open error instead of a read error
+        # if (!$oHostDbStandby->bogusHost())
+        # {
+        #     $oHostDbStandby->check($strComment, {iTimeout => 5, iExpectedExitStatus => ERROR_PATH_OPEN});
+        # }
+        # else
+        # {
+        #     $oHostDbStandby->check($strComment, {iTimeout => 5, iExpectedExitStatus => ERROR_FILE_READ});
+        # }
+        #
+        # # Remove the directory in pg_data location that is only readable by root
+        # executeTest("sudo rmdir ${strDir}");
+
+        # Confirm the check command runs without error on a standby (when a bogus host is not configured)
+        $oHostDbStandby->check('verify check command on standby', {strOptionalParam => $strBogusReset});
+
+        # Shutdown the standby before creating tablespaces (this will error since paths are different)
+        $oHostDbStandby->clusterStop({bIgnoreLogError => true});
+
         my $strAdhocBackup;
 
         # Execute stop and make sure the backup fails
         #---------------------------------------------------------------------------------------------------------------------------
         # Restart the cluster to check for any errors before continuing since the stop tests will definitely create errors and the
-        # logs will to be deleted to avoid causing issues further down the line. This test is not version specific so is run on only
-        # the expect version.
-        if ($bExpectVersion)
+        # logs will need to be deleted to avoid causing issues further down the line. This test is not version specific so is run on
+        # only one version.
+        if ($bNonVersionSpecific)
         {
             confess "test must be performed on posix storage" if $strStorage ne POSIX;
 
@@ -431,8 +417,8 @@ sub run
             'insert into test1_zeroed values (1);',
             {strDb => 'test1', bAutoCommit => true});
 
-        # Start a backup so the next backup has to restart it.  This test is not required for PostgreSQL >= 9.6 since backups
-        # are run in non-exclusive mode.
+        # Start a backup so the next backup has to restart it. This test is not required for PostgreSQL >= 9.6 since backups are run
+        # in non-exclusive mode.
         if ($oHostDbPrimary->pgVersion() >= PG_VERSION_93 && $oHostDbPrimary->pgVersion() < PG_VERSION_96)
         {
             $oHostDbPrimary->sqlSelectOne("select pg_start_backup('test backup that will cause an error', true)");
@@ -465,7 +451,7 @@ sub run
             {strOptionalParam => '--stop-auto --buffer-size=32768 --delta', iRepo => $iRepoTotal});
 
         # Ensure the check command runs properly with a tablespace
-        $oHostBackup->check( 'check command with tablespace', {iTimeout => 5, strOptionalParam => $strBogusReset});
+        $oHostBackup->check( 'check command with tablespace', {iTimeout => 10, strOptionalParam => $strBogusReset});
 
         # Setup the xid target
         #---------------------------------------------------------------------------------------------------------------------------
@@ -484,10 +470,7 @@ sub run
         $oHostDbPrimary->sqlExecute("update test set message = '$strNameMessage'", {bCommit => true});
         $oHostDbPrimary->sqlWalRotate();
 
-        if ($oHostDbPrimary->pgVersion() >= PG_VERSION_91)
-        {
-            $oHostDbPrimary->sqlExecute("select pg_create_restore_point('${strNameTarget}')");
-        }
+        $oHostDbPrimary->sqlExecute("select pg_create_restore_point('${strNameTarget}')");
 
         &log(INFO, "        name target is ${strNameTarget}");
 
@@ -507,7 +490,7 @@ sub run
         $oHostDbPrimary->sqlWalRotate();
 
         # Get the SHA1 and path of the table for the database that will not be restored
-        $strDb1TablePath =  $oHostDbPrimary->dbBasePath(). "/base/" .
+        $strDb1TablePath = $oHostDbPrimary->dbBasePath(). "/base/" .
             $oHostDbPrimary->sqlSelectOne("select oid from pg_database where datname='test1'") . "/" .
             $oHostDbPrimary->sqlSelectOne("select relfilenode from pg_class where relname='test1_zeroed'", {strDb => 'test1'});
         $strDb1TableSha1 = storageTest()->hashSize($strDb1TablePath);
@@ -538,8 +521,8 @@ sub run
         # Test that the first database has not been restored since --db-include did not include test1
         my ($strSHA1, $lSize) = storageTest()->hashSize($strDb1TablePath);
 
-        # Create a zeroed sparse file in the test directory that is the same size as the filenode.map.  We need to use the
-        # posix driver directly to do this because handles cannot be passed back from the C code.
+        # Create a zeroed sparse file in the test directory that is the same size as the filenode.map. We need to use the posix
+        # driver directly to do this because handles cannot be passed back from the C code.
         my $oStorageTrunc = new pgBackRestTest::Common::Storage($self->testPath(), new pgBackRestTest::Common::StoragePosix());
 
         my $strTestTable = $self->testPath() . "/testtable";
@@ -592,8 +575,8 @@ sub run
             confess &log(ASSERT, "no files found in tablespace path '${strTablespacePath}'");
         }
 
-        # This table should exist to prove that the tablespace was restored.  It has not been updated since it was created so it
-        # should not be created by any full page writes.  Once it is verified to exist it can be dropped.
+        # This table should exist to prove that the tablespace was restored. It has not been updated since it was created so it
+        # should not be created by any full page writes. Once it is verified to exist it can be dropped.
         $oHostDbPrimary->sqlSelectOneTest("select count(*) from test_exists", 0);
         $oHostDbPrimary->sqlExecute('drop table test_exists');
 
@@ -619,8 +602,7 @@ sub run
                 undef, $strFullBackup, {bForce => true, strType => CFGOPTVAL_RESTORE_TYPE_IMMEDIATE, strTargetAction => 'promote'});
 
             $oHostDbPrimary->clusterStart();
-            $oHostDbPrimary->sqlSelectOneTest(
-                'select message from test', ($bHostStandby ? $strStandbyMessage : $strFullMessage));
+            $oHostDbPrimary->sqlSelectOneTest('select message from test', ($strStandbyMessage));
         }
 
         # Restore (restore type = xid, inclusive)
@@ -636,8 +618,7 @@ sub run
 
         $oHostDbPrimary->restore(
             undef, $strIncrBackup,
-            {bForce => true, strType => CFGOPTVAL_RESTORE_TYPE_XID, strTarget => $strXidTarget,
-                strTargetAction => $oHostDbPrimary->pgVersion() >= PG_VERSION_91 ? 'promote' : undef,
+            {bForce => true, strType => CFGOPTVAL_RESTORE_TYPE_XID, strTarget => $strXidTarget, strTargetAction => 'promote',
                 strTargetTimeline => $oHostDbPrimary->pgVersion() >= PG_VERSION_12 ? 'current' : undef,
                 strOptionalParam => '--tablespace-map-all=../../tablespace', bTablespace => false,
                 iRepo => $iRepoTotal});
@@ -688,8 +669,7 @@ sub run
 
         $oHostDbPrimary->restore(
             undef, 'latest',
-            {bDelta => true, strType => CFGOPTVAL_RESTORE_TYPE_TIME, strTarget => $strTimeTarget,
-                strTargetAction => $oHostDbPrimary->pgVersion() >= PG_VERSION_91 ? 'promote' : undef,
+            {bDelta => true, strType => CFGOPTVAL_RESTORE_TYPE_TIME, strTarget => $strTimeTarget, strTargetAction => 'promote',
                 strTargetTimeline => $oHostDbPrimary->pgVersion() >= PG_VERSION_12 ? 'current' : undef,
                 strBackupExpected => $strFullBackup});
 
@@ -705,7 +685,7 @@ sub run
         $oHostDbPrimary->restore(
             undef, $strIncrBackup,
             {bDelta => true, strType => CFGOPTVAL_RESTORE_TYPE_XID, strTarget => $strXidTarget, bTargetExclusive => true,
-                strTargetAction => $oHostDbPrimary->pgVersion() >= PG_VERSION_91 ? 'promote' : undef,
+                strTargetAction => 'promote',
                 strTargetTimeline => $oHostDbPrimary->pgVersion() >= PG_VERSION_12 ? 'current' : undef,
                 iRepo => $iRepoTotal});
 
@@ -714,21 +694,18 @@ sub run
 
         # Restore (restore type = name)
         #---------------------------------------------------------------------------------------------------------------------------
-        if ($oHostDbPrimary->pgVersion() >= PG_VERSION_91)
-        {
-            &log(INFO, '    testing recovery type = ' . CFGOPTVAL_RESTORE_TYPE_NAME);
+        &log(INFO, '    testing recovery type = ' . CFGOPTVAL_RESTORE_TYPE_NAME);
 
-            $oHostDbPrimary->clusterStop();
+        $oHostDbPrimary->clusterStop();
 
-            $oHostDbPrimary->restore(
-                undef, 'latest',
-                {bDelta => true, bForce => true, strType => CFGOPTVAL_RESTORE_TYPE_NAME, strTarget => $strNameTarget,
-                    strTargetAction => 'promote',
-                    strTargetTimeline => $oHostDbPrimary->pgVersion() >= PG_VERSION_12 ? 'current' : undef});
+        $oHostDbPrimary->restore(
+            undef, 'latest',
+            {bDelta => true, bForce => true, strType => CFGOPTVAL_RESTORE_TYPE_NAME, strTarget => $strNameTarget,
+                strTargetAction => 'promote',
+                strTargetTimeline => $oHostDbPrimary->pgVersion() >= PG_VERSION_12 ? 'current' : undef});
 
-            $oHostDbPrimary->clusterStart();
-            $oHostDbPrimary->sqlSelectOneTest('select message from test', $strNameMessage);
-        }
+        $oHostDbPrimary->clusterStart();
+        $oHostDbPrimary->sqlSelectOneTest('select message from test', $strNameMessage);
 
         # Restore (restore type = default, timeline = created by type = xid, inclusive recovery)
         #---------------------------------------------------------------------------------------------------------------------------
@@ -736,9 +713,9 @@ sub run
 
         $oHostDbPrimary->clusterStop();
 
-        # The timeline to use for this test is subject to change based on tests being added or removed above.  The best thing
-        # would be to automatically grab the timeline after the restore, but since this test has been stable for a long time
-        # it does not seem worth the effort to automate.
+        # The timeline to use for this test is subject to change based on tests being added or removed above. The best thing would
+        # be to automatically grab the timeline after the restore, but since this test has been stable for a long time it does not
+        # seem worth the effort to automate.
         $oHostDbPrimary->restore(
             undef, $strIncrBackup,
             {bDelta => true, strType => CFGOPTVAL_RESTORE_TYPE_STANDBY, strTargetTimeline => 4, iRepo => $iRepoTotal});
@@ -751,9 +728,9 @@ sub run
         $oHostDbPrimary->clusterStop();
 
         # Stanza-delete --force without access to pgbackrest on database host. This test is not version specific so is run on only
-        # the expect version.
+        # one version.
         #---------------------------------------------------------------------------------------------------------------------------
-        if ($bExpectVersion)
+        if ($bNonVersionSpecific)
         {
             # Make sure this test has a backup host to work with
             confess "test must run with backup dst = " . HOST_BACKUP if !$bHostBackup;
