@@ -11,53 +11,49 @@ Harness for PostgreSQL Interface
 /***********************************************************************************************************************************
 Interface definition
 ***********************************************************************************************************************************/
-uint32_t hrnPgInterfaceCatalogVersion090(void);
-void hrnPgInterfaceControl090(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal090(PgWal pgWal, unsigned char *buffer);
-
-uint32_t hrnPgInterfaceCatalogVersion091(void);
-void hrnPgInterfaceControl091(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal091(PgWal pgWal, unsigned char *buffer);
-
-uint32_t hrnPgInterfaceCatalogVersion092(void);
-void hrnPgInterfaceControl092(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal092(PgWal pgWal, unsigned char *buffer);
-
 uint32_t hrnPgInterfaceCatalogVersion093(void);
-void hrnPgInterfaceControl093(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal093(PgWal pgWal, unsigned char *buffer);
+void hrnPgInterfaceControl093(unsigned int controlVersion, unsigned int crc, PgControl pgControl, unsigned char *buffer);
+void hrnPgInterfaceWal093(unsigned int magic, PgWal pgWal, unsigned char *buffer);
 
 uint32_t hrnPgInterfaceCatalogVersion094(void);
-void hrnPgInterfaceControl094(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal094(PgWal pgWal, unsigned char *buffer);
+void hrnPgInterfaceControl094(unsigned int controlVersion, unsigned int crc, PgControl pgControl, unsigned char *buffer);
+void hrnPgInterfaceWal094(unsigned int magic, PgWal pgWal, unsigned char *buffer);
 
 uint32_t hrnPgInterfaceCatalogVersion095(void);
-void hrnPgInterfaceControl095(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal095(PgWal pgWal, unsigned char *buffer);
+void hrnPgInterfaceControl095(unsigned int controlVersion, unsigned int crc, PgControl pgControl, unsigned char *buffer);
+void hrnPgInterfaceWal095(unsigned int magic, PgWal pgWal, unsigned char *buffer);
 
 uint32_t hrnPgInterfaceCatalogVersion096(void);
-void hrnPgInterfaceControl096(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal096(PgWal pgWal, unsigned char *buffer);
+void hrnPgInterfaceControl096(unsigned int controlVersion, unsigned int crc, PgControl pgControl, unsigned char *buffer);
+void hrnPgInterfaceWal096(unsigned int magic, PgWal pgWal, unsigned char *buffer);
 
 uint32_t hrnPgInterfaceCatalogVersion100(void);
-void hrnPgInterfaceControl100(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal100(PgWal pgWal, unsigned char *buffer);
+void hrnPgInterfaceControl100(unsigned int controlVersion, unsigned int crc, PgControl pgControl, unsigned char *buffer);
+void hrnPgInterfaceWal100(unsigned int magic, PgWal pgWal, unsigned char *buffer);
 
 uint32_t hrnPgInterfaceCatalogVersion110(void);
-void hrnPgInterfaceControl110(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal110(PgWal pgWal, unsigned char *buffer);
+void hrnPgInterfaceControl110(unsigned int controlVersion, unsigned int crc, PgControl pgControl, unsigned char *buffer);
+void hrnPgInterfaceWal110(unsigned int magic, PgWal pgWal, unsigned char *buffer);
 
 uint32_t hrnPgInterfaceCatalogVersion120(void);
-void hrnPgInterfaceControl120(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal120(PgWal pgWal, unsigned char *buffer);
+void hrnPgInterfaceControl120(unsigned int controlVersion, unsigned int crc, PgControl pgControl, unsigned char *buffer);
+void hrnPgInterfaceWal120(unsigned int magic, PgWal pgWal, unsigned char *buffer);
 
 uint32_t hrnPgInterfaceCatalogVersion130(void);
-void hrnPgInterfaceControl130(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal130(PgWal pgWal, unsigned char *buffer);
+void hrnPgInterfaceControl130(unsigned int controlVersion, unsigned int crc, PgControl pgControl, unsigned char *buffer);
+void hrnPgInterfaceWal130(unsigned int magic, PgWal pgWal, unsigned char *buffer);
 
 uint32_t hrnPgInterfaceCatalogVersion140(void);
-void hrnPgInterfaceControl140(PgControl pgControl, unsigned char *buffer);
-void hrnPgInterfaceWal140(PgWal pgWal, unsigned char *buffer);
+void hrnPgInterfaceControl140(unsigned int controlVersion, unsigned int crc, PgControl pgControl, unsigned char *buffer);
+void hrnPgInterfaceWal140(unsigned int magic, PgWal pgWal, unsigned char *buffer);
+
+uint32_t hrnPgInterfaceCatalogVersion150(void);
+void hrnPgInterfaceControl150(unsigned int controlVersion, unsigned int crc, PgControl pgControl, unsigned char *buffer);
+void hrnPgInterfaceWal150(unsigned int magic, PgWal pgWal, unsigned char *buffer);
+
+uint32_t hrnPgInterfaceCatalogVersion160(void);
+void hrnPgInterfaceControl160(unsigned int controlVersion, unsigned int crc, PgControl pgControl, unsigned char *buffer);
+void hrnPgInterfaceWal160(unsigned int magic, PgWal pgWal, unsigned char *buffer);
 
 typedef struct HrnPgInterface
 {
@@ -68,14 +64,28 @@ typedef struct HrnPgInterface
     unsigned int (*catalogVersion)(void);
 
     // Create pg_control
-    void (*control)(PgControl, unsigned char *);
+    void (*control)(unsigned int, unsigned int, PgControl, unsigned char *);
 
     // Create WAL header
-    void (*wal)(PgWal, unsigned char *);
+    void (*wal)(unsigned int, PgWal, unsigned char *);
 } HrnPgInterface;
 
 static const HrnPgInterface hrnPgInterface[] =
 {
+    {
+        .version = PG_VERSION_16,
+
+        .catalogVersion = hrnPgInterfaceCatalogVersion160,
+        .control = hrnPgInterfaceControl160,
+        .wal = hrnPgInterfaceWal160,
+    },
+    {
+        .version = PG_VERSION_15,
+
+        .catalogVersion = hrnPgInterfaceCatalogVersion150,
+        .control = hrnPgInterfaceControl150,
+        .wal = hrnPgInterfaceWal150,
+    },
     {
         .version = PG_VERSION_14,
 
@@ -139,26 +149,6 @@ static const HrnPgInterface hrnPgInterface[] =
         .control = hrnPgInterfaceControl093,
         .wal = hrnPgInterfaceWal093,
     },
-    {
-        .version = PG_VERSION_92,
-
-        .catalogVersion = hrnPgInterfaceCatalogVersion092,
-        .control = hrnPgInterfaceControl092,
-        .wal = hrnPgInterfaceWal092,
-    },
-    {
-        .version = PG_VERSION_91,
-        .catalogVersion = hrnPgInterfaceCatalogVersion091,
-        .control = hrnPgInterfaceControl091,
-        .wal = hrnPgInterfaceWal091,
-    },
-    {
-        .version = PG_VERSION_90,
-
-        .catalogVersion = hrnPgInterfaceCatalogVersion090,
-        .control = hrnPgInterfaceControl090,
-        .wal = hrnPgInterfaceWal090,
-    },
 };
 
 /***********************************************************************************************************************************
@@ -173,7 +163,7 @@ hrnPgInterfaceVersion(unsigned int pgVersion)
 
     const HrnPgInterface *result = NULL;
 
-    for (unsigned int interfaceIdx = 0; interfaceIdx < sizeof(hrnPgInterface) / sizeof(HrnPgInterface); interfaceIdx++)
+    for (unsigned int interfaceIdx = 0; interfaceIdx < LENGTH_OF(hrnPgInterface); interfaceIdx++)
     {
         if (hrnPgInterface[interfaceIdx].version == pgVersion)
         {
@@ -202,9 +192,11 @@ hrnPgCatalogVersion(unsigned int pgVersion)
 
 /**********************************************************************************************************************************/
 Buffer *
-hrnPgControlToBuffer(PgControl pgControl)
+hrnPgControlToBuffer(const unsigned int controlVersion, const unsigned int crc, PgControl pgControl)
 {
     FUNCTION_HARNESS_BEGIN();
+        FUNCTION_HARNESS_PARAM(UINT, controlVersion);
+        FUNCTION_HARNESS_PARAM(UINT, crc);
         FUNCTION_HARNESS_PARAM(PG_CONTROL, pgControl);
     FUNCTION_HARNESS_END();
 
@@ -213,8 +205,8 @@ hrnPgControlToBuffer(PgControl pgControl)
     // Set defaults if values are not passed
     pgControl.pageSize = pgControl.pageSize == 0 ? PG_PAGE_SIZE_DEFAULT : pgControl.pageSize;
     pgControl.walSegmentSize = pgControl.walSegmentSize == 0 ? PG_WAL_SEGMENT_SIZE_DEFAULT : pgControl.walSegmentSize;
-    pgControl.catalogVersion = pgControl.catalogVersion == 0 ?
-        hrnPgInterfaceVersion(pgControl.version)->catalogVersion() : pgControl.catalogVersion;
+    pgControl.catalogVersion =
+        pgControl.catalogVersion == 0 ? hrnPgInterfaceVersion(pgControl.version)->catalogVersion() : pgControl.catalogVersion;
     pgControl.systemId = pgControl.systemId < 100 ? hrnPgSystemId(pgControl.version) + pgControl.systemId : pgControl.systemId;
     pgControl.checkpoint = pgControl.checkpoint == 0 ? 1 : pgControl.checkpoint;
     pgControl.timeline = pgControl.timeline == 0 ? 1 : pgControl.timeline;
@@ -225,19 +217,20 @@ hrnPgControlToBuffer(PgControl pgControl)
     bufUsedSet(result, bufSize(result));
 
     // Generate pg_control
-    hrnPgInterfaceVersion(pgControl.version)->control(pgControl, bufPtr(result));
+    hrnPgInterfaceVersion(pgControl.version)->control(controlVersion, crc, pgControl, bufPtr(result));
 
     FUNCTION_HARNESS_RETURN(BUFFER, result);
 }
 
 /**********************************************************************************************************************************/
 void
-hrnPgWalToBuffer(PgWal pgWal, Buffer *walBuffer)
+hrnPgWalToBuffer(Buffer *const walBuffer, const unsigned int magic, PgWal pgWal)
 {
     FUNCTION_HARNESS_BEGIN();
-        FUNCTION_HARNESS_PARAM(PG_WAL, pgWal);
         FUNCTION_HARNESS_PARAM(BUFFER, walBuffer);
-    FUNCTION_TEST_END();
+        FUNCTION_HARNESS_PARAM(UINT, magic);
+        FUNCTION_HARNESS_PARAM(PG_WAL, pgWal);
+    FUNCTION_HARNESS_END();
 
     ASSERT(walBuffer != NULL);
 
@@ -250,7 +243,7 @@ hrnPgWalToBuffer(PgWal pgWal, Buffer *walBuffer)
         pgWal.systemId = hrnPgSystemId(pgWal.version) + pgWal.systemId;
 
     // Generate WAL
-    hrnPgInterfaceVersion(pgWal.version)->wal(pgWal, bufPtr(walBuffer));
+    hrnPgInterfaceVersion(pgWal.version)->wal(magic, pgWal, bufPtr(walBuffer));
 
     FUNCTION_HARNESS_RETURN_VOID();
 }
