@@ -6,11 +6,13 @@ Test Check Command
 #include "info/infoBackup.h"
 #include "postgres/version.h"
 #include "storage/helper.h"
+#include "storage/posix/storage.h"
 
 #include "common/harnessConfig.h"
 #include "common/harnessInfo.h"
 #include "common/harnessPostgres.h"
 #include "common/harnessPq.h"
+#include "common/harnessStorage.h"
 
 /***********************************************************************************************************************************
 Test Run
@@ -20,11 +22,190 @@ testRun(void)
 {
     FUNCTION_HARNESS_VOID();
 
+    // Create default storage object for testing
+    const Storage *const storageTest = storagePosixNewP(TEST_PATH_STR, .write = true);
+
     // PQfinish() is strictly checked
     harnessPqScriptStrictSet(true);
 
     StringList *argList = strLstNew();
 
+    // *****************************************************************************************************************************
+    if (testBegin("checkReport()"))
+    {
+        // Common config
+        HRN_STORAGE_PUT_Z(
+            storageTest, "pgbackrest.conf",
+            "[global]\n"
+            "repo1-path=" TEST_PATH "/repo1\n"
+            "repo1-block=y\n"
+            "no-repo1-block=bogus\n"
+            "bogus=y\n"
+            "\n"
+            "[global:backup]\n"
+            "start-fast=y\n"
+            "reset-start-fast=bogus\n"
+            "\n"
+            "[test2]\n"
+            "pg1-path=" TEST_PATH "/test2-pg1\n"
+            "recovery-option=key1=value1\n"
+            "recovery-option=key2=value2\n"
+            "\n"
+            "[test1]\n"
+            "pg1-path=" TEST_PATH "/test1-pg1\n");
+
+        StringList *const argListCommon = strLstNew();
+        hrnCfgArgRawZ(argListCommon, cfgOptConfig, TEST_PATH "/pgbackrest.conf");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("no env, no config");
+        {
+            TEST_RESULT_STR_Z(
+                checkReport(),
+                // {uncrustify_off - indentation}
+                "{"
+                    "\"cfg\":{"
+                        "\"env\":{},"
+                        "\"file\":null"
+                    "}"
+                "}",
+                // {uncrustify_on}
+                "render");
+        }
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("report on stdout");
+        {
+            argList = strLstNew();
+            hrnCfgArgRawBool(argList, cfgOptReport, true);
+            HRN_CFG_LOAD(cfgCmdCheck, argList);
+
+            // Redirect stdout to a file
+            int stdoutSave = dup(STDOUT_FILENO);
+            const String *stdoutFile = STRDEF(TEST_PATH "/stdout.info");
+
+            THROW_ON_SYS_ERROR(freopen(strZ(stdoutFile), "w", stdout) == NULL, FileWriteError, "unable to reopen stdout");
+
+            // Not in a test wrapper to avoid writing to stdout
+            cmdCheck();
+
+            // Restore normal stdout
+            dup2(stdoutSave, STDOUT_FILENO);
+
+            // Check output of info command stored in file
+            TEST_STORAGE_GET(
+                storageTest, strZ(stdoutFile),
+                // {uncrustify_off - indentation}
+                "{"
+                    "\"cfg\":{"
+                        "\"env\":{},"
+                        "\"file\":null"
+                    "}"
+                "}",
+                // {uncrustify_on}
+                .remove = true);
+        }
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("env and config");
+        {
+            hrnCfgEnvRawZ(cfgOptBufferSize, "64KiB");
+            setenv(PGBACKREST_ENV "BOGUS", "bogus", true);
+            setenv(PGBACKREST_ENV "NO_ONLINE", "bogus", true);
+            setenv(PGBACKREST_ENV "DB_INCLUDE", "db1:db2", true);
+            setenv(PGBACKREST_ENV "RESET_COMPRESS_TYPE", "bogus", true);
+
+            hrnCfgLoad(cfgCmdCheck, argListCommon, (HrnCfgLoadParam){.log = true});
+
+            TEST_RESULT_STR_Z(
+                checkReport(),
+                // {uncrustify_off - indentation}
+                "{"
+                    "\"cfg\":{"
+                        "\"env\":{"
+                            "\"PGBACKREST_BOGUS\":{"
+                                "\"warn\":\"invalid option\""
+                            "},"
+                            "\"PGBACKREST_BUFFER_SIZE\":{"
+                                "\"val\":\"64KiB\""
+                            "},"
+                            "\"PGBACKREST_DB_INCLUDE\":{"
+                                "\"val\":["
+                                    "\"db1\","
+                                    "\"db2\""
+                                "]"
+                            "},"
+                            "\"PGBACKREST_NO_ONLINE\":{"
+                                "\"val\":\"bogus\","
+                                "\"warn\":\"invalid negate option\""
+                            "},"
+                            "\"PGBACKREST_RESET_COMPRESS_TYPE\":{"
+                                "\"val\":\"bogus\","
+                                "\"warn\":\"invalid reset option\""
+                            "}"
+                        "},"
+                        "\"file\":{"
+                            "\"global\":{"
+                                "\"bogus\":{"
+                                    "\"warn\":\"invalid option\""
+                                "},"
+                                "\"no-repo1-block\":{"
+                                    "\"val\":\"bogus\","
+                                    "\"warn\":\"invalid negate option\""
+                                "},"
+                                "\"repo1-block\":{"
+                                    "\"val\":\"y\""
+                                "},"
+                                "\"repo1-path\":{"
+                                    "\"val\":\"" TEST_PATH "/repo1\""
+                                "}"
+                            "},"
+                            "\"global:backup\":{"
+                                "\"reset-start-fast\":"
+                                "{"
+                                    "\"val\":\"bogus\","
+                                    "\"warn\":\"invalid reset option\""
+                                "},"
+                                "\"start-fast\":{"
+                                    "\"val\":\"y\""
+                                "}"
+                            "},"
+                            "\"test1\":{"
+                                "\"pg1-path\":{"
+                                    "\"val\":\"" TEST_PATH "/test1-pg1\""
+                                "}"
+                            "},"
+                            "\"test2\":{"
+                                "\"pg1-path\":{"
+                                    "\"val\":\"" TEST_PATH "/test2-pg1\""
+                                "},"
+                                "\"recovery-option\":{"
+                                    "\"val\":["
+                                        "\"key1=value1\","
+                                        "\"key2=value2\""
+                                    "]"
+                                "}"
+                            "}"
+                        "}"
+                    "}"
+                "}",
+                // {uncrustify_on}
+                "render");
+
+            TEST_RESULT_LOG(
+                "P00   WARN: environment contains invalid option 'bogus'\n"
+                "P00   WARN: environment contains invalid negate option 'no-online'\n"
+                "P00   WARN: environment contains invalid reset option 'reset-compress-type'\n"
+                "P00   WARN: configuration file contains negate option 'no-repo1-block'\n"
+                "P00   WARN: configuration file contains invalid option 'bogus'");
+
+            unsetenv(PGBACKREST_ENV "BOGUS");
+            unsetenv(PGBACKREST_ENV "NO_ONLINE");
+            unsetenv(PGBACKREST_ENV "DB_INCLUDE");
+            unsetenv(PGBACKREST_ENV "RESET_COMPRESS_TYPE");
+        }
+    }
+
     // *****************************************************************************************************************************
     if (testBegin("cmdCheck()"))
     {
@@ -272,9 +453,13 @@ testRun(void)
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("multi-repo - primary database only, WAL not found");
 
+        HRN_STORAGE_PUT_Z(
+            storageTest, "pgbackrest.conf",
+            "[test1]\n"
+            "pg1-path=" TEST_PATH "/pg\n");
+
         argList = strLstNew();
-        hrnCfgArgRawZ(argList, cfgOptStanza, "test1");
-        hrnCfgArgRawZ(argList, cfgOptPgPath, TEST_PATH "/pg");
+        hrnCfgArgRawZ(argList, cfgOptConfig, TEST_PATH "/pgbackrest.conf");
         hrnCfgArgRawZ(argList, cfgOptRepoPath, TEST_PATH "/repo");
         hrnCfgArgKeyRawZ(argList, cfgOptRepoPath, 2, TEST_PATH "/repo2");
         hrnCfgArgRawZ(argList, cfgOptArchiveTimeout, ".5");
@@ -282,7 +467,7 @@ testRun(void)
 
         // Create stanza files on repo2
         HRN_INFO_PUT(
-            storageRepoIdxWrite(1), INFO_ARCHIVE_PATH_FILE,
+            storageTest, "repo2/archive/test1/" INFO_ARCHIVE_FILE,
             "[db]\n"
             "db-id=1\n"
             "db-system-id=" HRN_PG_SYSTEMID_15_Z "\n"
@@ -291,7 +476,7 @@ testRun(void)
             "[db:history]\n"
             "1={\"db-id\":" HRN_PG_SYSTEMID_15_Z ",\"db-version\":\"15\"}\n");
         HRN_INFO_PUT(
-            storageRepoIdxWrite(1), INFO_BACKUP_PATH_FILE,
+            storageTest, "repo2/backup/test1/" INFO_BACKUP_FILE,
             "[db]\n"
             "db-catalog-version=202209061\n"
             "db-control-version=1300\n"
@@ -320,13 +505,35 @@ testRun(void)
             "HINT: check the PostgreSQL server log for errors.\n"
             "HINT: run the 'start' command if the stanza was previously stopped.");
         TEST_RESULT_LOG(
+            "P00   INFO: check stanza 'test1'\n"
             "P00   INFO: check repo1 configuration (primary)\n"
             "P00   INFO: check repo2 configuration (primary)\n"
             "P00   INFO: check repo1 archive for WAL (primary)");
 
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("no stanzas in config file");
+
+        HRN_STORAGE_PUT_Z(
+            storageTest, "pgbackrest.conf",
+            "[test1]\n");
+        HRN_CFG_LOAD(cfgCmdCheck, argList);
+
+        TEST_RESULT_VOID(cmdCheck(), "check");
+        TEST_RESULT_LOG(
+            "P00   WARN: no stanzas found to check\n"
+            "            HINT: are there non-empty stanza sections in the configuration?");
+
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("multi-repo - WAL segment switch performed once for all repos");
 
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test1");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, TEST_PATH "/pg");
+        hrnCfgArgRawZ(argList, cfgOptRepoPath, TEST_PATH "/repo");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoPath, 2, TEST_PATH "/repo2");
+        hrnCfgArgRawZ(argList, cfgOptArchiveTimeout, ".5");
+        HRN_CFG_LOAD(cfgCmdCheck, argList);
+
         // Create WAL segment
         Buffer *buffer = bufNew(16 * 1024 * 1024);
         memset(bufPtr(buffer), 0, bufSize(buffer));
