@@ -14,17 +14,18 @@ Test GCS Storage
 Constants
 ***********************************************************************************************************************************/
 #define TEST_ENDPOINT                                               "storage.googleapis.com"
-    STRING_STATIC(TEST_ENDPOINT_STR,                                TEST_ENDPOINT);
+STRING_STATIC(TEST_ENDPOINT_STR,                                    TEST_ENDPOINT);
 #define TEST_PORT                                                   ((unsigned int)443)
 #define TEST_TIMEOUT                                                5000
 #define TEST_CHUNK_SIZE                                             16
 #define TEST_BUCKET                                                 "bucket"
-    STRING_STATIC(TEST_BUCKET_STR,                                  TEST_BUCKET);
+STRING_STATIC(TEST_BUCKET_STR,                                      TEST_BUCKET);
 #define TEST_KEY_FILE                                               TEST_PATH "/key.json"
-    STRING_STATIC(TEST_KEY_FILE_STR,                                TEST_KEY_FILE);
+STRING_STATIC(TEST_KEY_FILE_STR,                                    TEST_KEY_FILE);
 #define TEST_TOKEN                                                  "X X"
-    STRING_STATIC(TEST_TOKEN_STR,                                   TEST_TOKEN);
+STRING_STATIC(TEST_TOKEN_STR,                                       TEST_TOKEN);
 
+// {uncrustify_off - comment inside string}
 #define TEST_KEY                                                                                                                   \
     "{\n"                                                                                                                          \
     "\"type\": \"service_account\",\n"                                                                                             \
@@ -60,6 +61,7 @@ Constants
     "\"client_email\": \"service@project.iam.gserviceaccount.com\",\n"                                                             \
     "\"token_uri\": \"https://%s:%u/token\"\n"                                                                                     \
     "}\n"
+// {uncrustify_on}
 
 /***********************************************************************************************************************************
 Helper to build test requests
@@ -73,6 +75,7 @@ typedef struct TestRequestParam
     const char *object;
     const char *query;
     const char *contentRange;
+    const char *contentType;
     const char *content;
     const char *range;
 } TestRequestParam;
@@ -108,6 +111,10 @@ testRequest(IoWrite *write, const char *verb, TestRequestParam param)
     if (param.contentRange != NULL)
         strCatFmt(request, "content-range:bytes %s\r\n", param.contentRange);
 
+    // Add content-type
+    if (param.contentType != NULL)
+        strCatFmt(request, "content-type:%s\r\n", param.contentType);
+
     // Add host
     strCatFmt(request, "host:%s\r\n", strZ(hrnServerHost()));
 
@@ -173,8 +180,8 @@ testResponse(IoWrite *write, TestResponseParam param)
         strCatFmt(
             response,
             "content-length:%zu\r\n"
-                "\r\n"
-                "%s",
+            "\r\n"
+            "%s",
             strlen(param.content), param.content);
     }
     else
@@ -223,7 +230,7 @@ testRun(void)
         TEST_RESULT_STR_Z(storage->path, "/repo", "check path");
         TEST_RESULT_STR(((StorageGcs *)storageDriver(storage))->bucket, TEST_BUCKET_STR, "check bucket");
         TEST_RESULT_STR_Z(((StorageGcs *)storageDriver(storage))->endpoint, "storage.googleapis.com", "check endpoint");
-        TEST_RESULT_UINT(((StorageGcs *)storageDriver(storage))->chunkSize, STORAGE_GCS_CHUNKSIZE_DEFAULT, "check chunk size");
+        TEST_RESULT_UINT(((StorageGcs *)storageDriver(storage))->chunkSize, 4 * 1024 * 1024, "check chunk size");
         TEST_RESULT_STR(((StorageGcs *)storageDriver(storage))->token, TEST_TOKEN_STR, "check token");
         TEST_RESULT_BOOL(storageFeature(storage, storageFeaturePath), false, "check path feature");
     }
@@ -243,7 +250,7 @@ testRun(void)
             (StorageGcs *)storageDriver(
                 storageGcsNew(
                     STRDEF("/repo"), false, NULL, TEST_BUCKET_STR, storageGcsKeyTypeService, TEST_KEY_FILE_STR, TEST_CHUNK_SIZE,
-                    TEST_ENDPOINT_STR, TEST_TIMEOUT, true, NULL, NULL)),
+                    NULL, TEST_ENDPOINT_STR, TEST_TIMEOUT, true, NULL, NULL)),
             "read-only gcs storage - service key");
         TEST_RESULT_STR_Z(httpUrlHost(storage->authUrl), "test.com", "check host");
         TEST_RESULT_STR_Z(httpUrlPath(storage->authUrl), "/token", "check path");
@@ -268,7 +275,7 @@ testRun(void)
             (StorageGcs *)storageDriver(
                 storageGcsNew(
                     STRDEF("/repo"), true, NULL, TEST_BUCKET_STR, storageGcsKeyTypeService, TEST_KEY_FILE_STR, TEST_CHUNK_SIZE,
-                    TEST_ENDPOINT_STR, TEST_TIMEOUT, true, NULL, NULL)),
+                    NULL, TEST_ENDPOINT_STR, TEST_TIMEOUT, true, NULL, NULL)),
             "read/write gcs storage - service key");
 
         TEST_RESULT_STR_Z(
@@ -289,20 +296,20 @@ testRun(void)
 
         HRN_FORK_BEGIN()
         {
-            HRN_FORK_CHILD_BEGIN(.prefix = "gcs server", .timeout = 5000)
+            HRN_FORK_CHILD_BEGIN(.prefix = "gcs server", .timeout = 10000)
             {
                 TEST_RESULT_VOID(hrnServerRunP(HRN_FORK_CHILD_READ(), hrnServerProtocolTls, .port = testPort), "gcs server run");
             }
             HRN_FORK_CHILD_END();
 
-            HRN_FORK_CHILD_BEGIN(.prefix = "auth server", .timeout = 5000)
+            HRN_FORK_CHILD_BEGIN(.prefix = "auth server", .timeout = 10000)
             {
                 TEST_RESULT_VOID(
                     hrnServerRunP(HRN_FORK_CHILD_READ(), hrnServerProtocolTls, .port = testPortAuth), "auth server run");
             }
             HRN_FORK_CHILD_END();
 
-            HRN_FORK_CHILD_BEGIN(.prefix = "meta server", .timeout = 15000)
+            HRN_FORK_CHILD_BEGIN(.prefix = "meta server", .timeout = 30000)
             {
                 TEST_RESULT_VOID(
                     hrnServerRunP(HRN_FORK_CHILD_READ(), hrnServerProtocolSocket, .port = testPortMeta), "meta server run");
@@ -368,7 +375,11 @@ testRun(void)
 
                 storageGcsRequestP(
                     (StorageGcs *)storageDriver(storage), HTTP_VERB_POST_STR, .noBucket = true,
-                    .content = BUFSTR(jsonFromKv(kvPut(kvNew(), GCS_JSON_NAME_VAR, VARSTRDEF("bucket")))));
+                    .content = BUFSTR(
+                        jsonWriteResult(
+                            jsonWriteObjectEnd(
+                                jsonWriteStr(
+                                    jsonWriteKeyZ(jsonWriteObjectBegin(jsonWriteNewP()), GCS_JSON_NAME), STRDEF("bucket"))))));
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("auth error");
@@ -428,6 +439,8 @@ testRun(void)
 
                 StringList *argListAuto = strLstDup(argList);
                 hrnCfgArgRawStrId(argListAuto, cfgOptRepoGcsKeyType, storageGcsKeyTypeAuto);
+                hrnCfgArgRawZ(argListAuto, cfgOptRepoStorageTag, "Key1=Value1");
+                hrnCfgArgRawZ(argListAuto, cfgOptRepoStorageTag, " Key 2= Value 2");
                 HRN_CFG_LOAD(cfgCmdArchivePush, argListAuto);
 
                 TEST_ASSIGN(storage, storageRepoGet(0, true), "get repo storage");
@@ -440,6 +453,10 @@ testRun(void)
                 // Tests need the chunk size to be 16
                 ((StorageGcs *)storageDriver(storage))->chunkSize = 16;
 
+                // Store tags and set to NULL
+                const Buffer *tag = ((StorageGcs *)storageDriver(storage))->tag;
+                ((StorageGcs *)storageDriver(storage))->tag = NULL;
+
                 hrnServerScriptAccept(service);
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -536,6 +553,7 @@ testRun(void)
                 TEST_RESULT_STR_Z(storageWriteName(write), "/file.txt", "check file name");
                 TEST_RESULT_BOOL(storageWriteSyncFile(write), true, "file is synced");
                 TEST_RESULT_BOOL(storageWriteSyncPath(write), true, "path is synced");
+                TEST_RESULT_BOOL(storageWriteTruncate(write), true, "file will be truncated");
 
                 TEST_RESULT_VOID(storageWriteGcsClose(write->driver), "close file again");
 
@@ -575,9 +593,31 @@ testRun(void)
                 TEST_ERROR(storagePutP(write, NULL), FormatError, "expected size 55 for '/file.txt' but actual is 0");
 
                 // -----------------------------------------------------------------------------------------------------------------
-                TEST_TITLE("write file in chunks with nothing left over on close");
+                TEST_TITLE("write zero-length file (with tags)");
 
-                testRequestP(service, HTTP_VERB_POST, .upload = true, .query = "name=file.txt&uploadType=resumable");
+                testRequestP(
+                    service, HTTP_VERB_POST, .upload = true, .query = "name=file.txt&uploadType=resumable",
+                    .contentType = "application/json", .content = "{\"metadata\":{\" Key 2\":\" Value 2\",\"Key1\":\"Value1\"}}");
+                testResponseP(service, .header = "x-guploader-uploadid:ulid3");
+
+                testRequestP(
+                    service, HTTP_VERB_PUT, .upload = true, .noAuth = true,
+                    .query = "fields=md5Hash%2Csize&name=file.txt&uploadType=resumable&upload_id=ulid3", .contentRange = "*/0");
+                testResponseP(service, .content = "{\"md5Hash\":\"1B2M2Y8AsgTpgAmY7PhCfg==\",\"size\":\"0\"}");
+
+                ((StorageGcs *)storageDriver(storage))->tag = tag;
+
+                TEST_ASSIGN(write, storageNewWriteP(storage, STRDEF("file.txt")), "new write");
+                TEST_RESULT_VOID(storagePutP(write, NULL), "write");
+
+                ((StorageGcs *)storageDriver(storage))->tag = NULL;
+
+                // -----------------------------------------------------------------------------------------------------------------
+                TEST_TITLE("write file in chunks with nothing left over on close (with tags)");
+
+                testRequestP(
+                    service, HTTP_VERB_POST, .upload = true, .query = "name=file.txt&uploadType=resumable",
+                    .contentType = "application/json", .content = "{\"metadata\":{\" Key 2\":\" Value 2\",\"Key1\":\"Value1\"}}");
                 testResponseP(service, .header = "x-guploader-uploadid:ulid1");
 
                 testRequestP(
@@ -588,13 +628,22 @@ testRun(void)
 
                 testRequestP(
                     service, HTTP_VERB_PUT, .upload = true, .noAuth = true,
-                    .query = "fields=md5Hash%2Csize&name=file.txt&uploadType=resumable&upload_id=ulid1", .contentRange = "16-31/32",
+                    .query = "name=file.txt&uploadType=resumable&upload_id=ulid1", .contentRange = "16-31/*",
                     .content = "7890123456789012");
+                testResponseP(service, .code = 308);
+
+                testRequestP(
+                    service, HTTP_VERB_PUT, .upload = true, .noAuth = true,
+                    .query = "fields=md5Hash%2Csize&name=file.txt&uploadType=resumable&upload_id=ulid1", .contentRange = "*/32");
                 testResponseP(service, .content = "{\"md5Hash\":\"dnF5x6K/8ZZRzpfSlMMM+w==\",\"size\":\"32\"}");
 
+                ((StorageGcs *)storageDriver(storage))->tag = tag;
+
                 TEST_ASSIGN(write, storageNewWriteP(storage, STRDEF("file.txt")), "new write");
                 TEST_RESULT_VOID(storagePutP(write, BUFSTRDEF("12345678901234567890123456789012")), "write");
 
+                ((StorageGcs *)storageDriver(storage))->tag = NULL;
+
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("write file in chunks with something left over on close");
 
@@ -709,22 +758,15 @@ testRun(void)
                         "  ]"
                         "}");
 
-                HarnessStorageInfoListCallbackData callbackData =
-                {
-                    .content = strNew(),
-                };
-
                 TEST_ERROR(
-                    storageInfoListP(storage, STRDEF("/"), hrnStorageInfoListCallback, NULL, .errorOnMissing = true),
-                    AssertError, "assertion '!param.errorOnMissing || storageFeature(this, storageFeaturePath)' failed");
+                    storageNewItrP(storage, STRDEF("/"), .errorOnMissing = true), AssertError,
+                    "assertion '!param.errorOnMissing || storageFeature(this, storageFeaturePath)' failed");
 
-                TEST_RESULT_VOID(
-                    storageInfoListP(storage, STRDEF("/path/to"), hrnStorageInfoListCallback, &callbackData), "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "test_path {path}\n"
-                    "test_file {file, s=787, t=1255369830}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/path/to",
+                    "test_file {s=787, t=1255369830}\n"
+                    "test_path/\n",
+                    .level = storageInfoLevelBasic, .noRecurse = true);
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list exists level");
@@ -744,17 +786,11 @@ testRun(void)
                         "  ]"
                         "}");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        storage, STRDEF("/"), hrnStorageInfoListCallback, &callbackData, .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "path1 {}\n"
-                    "test1.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/",
+                    "path1/\n"
+                    "test1.txt\n",
+                    .noRecurse = true);
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list a file in root with expression");
@@ -772,17 +808,10 @@ testRun(void)
                         "  ]"
                         "}");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        storage, STRDEF("/"), hrnStorageInfoListCallback, &callbackData, .expression = STRDEF("^test.*$"),
-                        .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "test1.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/",
+                    "test1.txt\n",
+                    .noRecurse = true, .expression = "^test.*$");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list files with continuation");
@@ -825,20 +854,14 @@ testRun(void)
                         "  ]"
                         "}");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        storage, STRDEF("/path/to"), hrnStorageInfoListCallback, &callbackData, .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "path1 {}\n"
-                    "test1.txt {}\n"
-                    "test2.txt {}\n"
-                    "path2 {}\n"
-                    "test3.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/path/to",
+                    "path1/\n"
+                    "path2/\n"
+                    "test1.txt\n"
+                    "test2.txt\n"
+                    "test3.txt\n",
+                    .noRecurse = true);
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list files with expression");
@@ -867,19 +890,12 @@ testRun(void)
                         "  ]"
                         "}");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        storage, STRDEF("/path/to"), hrnStorageInfoListCallback, &callbackData, .expression = STRDEF("^test(1|3)"),
-                        .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "test1.path {}\n"
-                    "test1.txt {}\n"
-                    "test3.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    storage, "/path/to",
+                    "test1.path\n"
+                    "test1.txt\n"
+                    "test3.txt\n",
+                    .level = storageInfoLevelExists, .noRecurse = true, .expression = "^test(1|3)");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("switch to token auth");
