@@ -31,6 +31,7 @@ typedef struct TestRequestParam
     const char *kms;
     const char *ttl;
     const char *token;
+    const char *tag;
 } TestRequestParam;
 
 #define testRequestP(write, s3, verb, path, ...)                                                                                   \
@@ -81,6 +82,9 @@ testRequest(IoWrite *write, Storage *s3, const char *verb, const char *path, Tes
         if (param.kms != NULL)
             strCatZ(request, ";x-amz-server-side-encryption;x-amz-server-side-encryption-aws-kms-key-id");
 
+        if (param.tag != NULL)
+            strCatZ(request, ";x-amz-tagging");
+
         strCatZ(request, ",Signature=????????????????????????????????????????????????????????????????\r\n");
     }
 
@@ -116,7 +120,7 @@ testRequest(IoWrite *write, Storage *s3, const char *verb, const char *path, Tes
         strCatFmt(
             request,
             "x-amz-content-sha256:%s\r\n"
-                "x-amz-date:????????T??????Z" "\r\n",
+            "x-amz-date:????????T??????Z" "\r\n",
             param.content == NULL ?
                 HASH_TYPE_SHA256_ZERO : strZ(strNewEncode(encodingHex, cryptoHashOne(hashTypeSha256, BUFSTRZ(param.content)))));
 
@@ -132,6 +136,10 @@ testRequest(IoWrite *write, Storage *s3, const char *verb, const char *path, Tes
         strCatFmt(request, "x-amz-server-side-encryption-aws-kms-key-id:%s\r\n", param.kms);
     }
 
+    // Add tags
+    if (param.tag != NULL)
+        strCatFmt(request, "x-amz-tagging:%s\r\n", param.tag);
+
     // Add metadata token
     if (param.token != NULL)
         strCatFmt(request, "x-aws-ec2-metadata-token:%s\r\n", param.token);
@@ -195,8 +203,8 @@ testResponse(IoWrite *write, TestResponseParam param)
         strCatFmt(
             response,
             "content-length:%zu\r\n"
-                "\r\n"
-                "%s",
+            "\r\n"
+            "%s",
             strlen(param.content), param.content);
     }
     else
@@ -295,11 +303,12 @@ testRun(void)
         TEST_RESULT_STR(driver->securityToken, NULL, "check security token");
 
         TEST_RESULT_VOID(FUNCTION_LOG_OBJECT_FORMAT(driver->httpClient, httpClientToLog, logBuf, sizeof(logBuf)), "httpClientToLog");
-        TEST_RESULT_Z(logBuf,
+        TEST_RESULT_Z(
+            logBuf,
             zNewFmt(
                 "{ioClient: {type: tls, driver: {ioClient: {type: socket, driver: {host: bucket.s3.amazonaws.com, port: 443"
-                    ", timeoutConnect: 60000, timeoutSession: 60000}}, timeoutConnect: 60000, timeoutSession: 60000"
-                    ", verifyPeer: %s}}, reusable: 0, timeout: 60000}",
+                ", timeoutConnect: 60000, timeoutSession: 60000}}, timeoutConnect: 60000, timeoutSession: 60000"
+                ", verifyPeer: %s}}, reusable: 0, timeout: 60000}",
                 cvtBoolToConstZ(TEST_IN_CONTAINER)),
             "check http client");
 
@@ -316,9 +325,9 @@ testRun(void)
             "generate authorization");
         TEST_RESULT_STR_Z(
             httpHeaderGet(header, STRDEF("authorization")),
-            "AWS4-HMAC-SHA256 Credential=AKIAIOSFODNN7EXAMPLE/20170606/us-east-1/s3/aws4_request,"
-                "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
-                "Signature=cb03bf1d575c1f8904dabf0e573990375340ab293ef7ad18d049fc1338fd89b3",
+            "AWS4-HMAC-SHA256 Credential=AKIAIOSFODNN7EXAMPLE/20170606/us-east-1/s3/aws4_request"
+            ",SignedHeaders=host;x-amz-content-sha256;x-amz-date"
+            ",Signature=cb03bf1d575c1f8904dabf0e573990375340ab293ef7ad18d049fc1338fd89b3",
             "check authorization header");
 
         // Test again to be sure cache signing key is used
@@ -330,9 +339,9 @@ testRun(void)
             "generate authorization");
         TEST_RESULT_STR_Z(
             httpHeaderGet(header, STRDEF("authorization")),
-            "AWS4-HMAC-SHA256 Credential=AKIAIOSFODNN7EXAMPLE/20170606/us-east-1/s3/aws4_request,"
-                "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
-                "Signature=cb03bf1d575c1f8904dabf0e573990375340ab293ef7ad18d049fc1338fd89b3",
+            "AWS4-HMAC-SHA256 Credential=AKIAIOSFODNN7EXAMPLE/20170606/us-east-1/s3/aws4_request"
+            ",SignedHeaders=host;x-amz-content-sha256;x-amz-date"
+            ",Signature=cb03bf1d575c1f8904dabf0e573990375340ab293ef7ad18d049fc1338fd89b3",
             "check authorization header");
         TEST_RESULT_BOOL(driver->signingKey == lastSigningKey, true, "check signing key was reused");
 
@@ -345,9 +354,9 @@ testRun(void)
             "generate authorization");
         TEST_RESULT_STR_Z(
             httpHeaderGet(header, STRDEF("authorization")),
-            "AWS4-HMAC-SHA256 Credential=AKIAIOSFODNN7EXAMPLE/20180814/us-east-1/s3/aws4_request,"
-                "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
-                "Signature=d0fa9c36426eb94cdbaf287a7872c7a3b6c913f523163d0d7debba0758e36f49",
+            "AWS4-HMAC-SHA256 Credential=AKIAIOSFODNN7EXAMPLE/20180814/us-east-1/s3/aws4_request"
+            ",SignedHeaders=host;x-amz-content-sha256;x-amz-date"
+            ",Signature=d0fa9c36426eb94cdbaf287a7872c7a3b6c913f523163d0d7debba0758e36f49",
             "check authorization header");
         TEST_RESULT_BOOL(driver->signingKey != lastSigningKey, true, "check signing key was regenerated");
 
@@ -366,11 +375,12 @@ testRun(void)
         TEST_RESULT_STR(driver->securityToken, securityToken, "check security token");
         TEST_RESULT_VOID(
             FUNCTION_LOG_OBJECT_FORMAT(driver->httpClient, httpClientToLog, logBuf, sizeof(logBuf)), "httpClientToLog");
-        TEST_RESULT_Z(logBuf,
+        TEST_RESULT_Z(
+            logBuf,
             zNewFmt(
                 "{ioClient: {type: tls, driver: {ioClient: {type: socket, driver: {host: bucket.custom.endpoint, port: 333"
-                    ", timeoutConnect: 60000, timeoutSession: 60000}}, timeoutConnect: 60000, timeoutSession: 60000"
-                    ", verifyPeer: %s}}, reusable: 0, timeout: 60000}",
+                ", timeoutConnect: 60000, timeoutSession: 60000}}, timeoutConnect: 60000, timeoutSession: 60000"
+                ", verifyPeer: %s}}, reusable: 0, timeout: 60000}",
                 cvtBoolToConstZ(TEST_IN_CONTAINER)),
             "check http client");
 
@@ -383,9 +393,9 @@ testRun(void)
             "generate authorization");
         TEST_RESULT_STR_Z(
             httpHeaderGet(header, STRDEF("authorization")),
-            "AWS4-HMAC-SHA256 Credential=AKIAIOSFODNN7EXAMPLE/20170606/us-east-1/s3/aws4_request,"
-                "SignedHeaders=host;x-amz-content-sha256;x-amz-date;x-amz-security-token,"
-                "Signature=85278841678ccbc0f137759265030d7b5e237868dd36eea658426b18344d1685",
+            "AWS4-HMAC-SHA256 Credential=AKIAIOSFODNN7EXAMPLE/20170606/us-east-1/s3/aws4_request"
+            ",SignedHeaders=host;x-amz-content-sha256;x-amz-date;x-amz-security-token"
+            ",Signature=85278841678ccbc0f137759265030d7b5e237868dd36eea658426b18344d1685",
             "check authorization header");
     }
 
@@ -482,6 +492,8 @@ testRun(void)
                 hrnCfgArgRaw(argList, cfgOptRepoS3Role, credRole);
                 hrnCfgArgRawStrId(argList, cfgOptRepoS3KeyType, storageS3KeyTypeAuto);
                 hrnCfgArgRawZ(argList, cfgOptRepoS3KmsKeyId, "kmskey1");
+                hrnCfgArgRawZ(argList, cfgOptRepoStorageTag, "Key1=Value1");
+                hrnCfgArgRawZ(argList, cfgOptRepoStorageTag, " Key 2= Value 2");
                 HRN_CFG_LOAD(cfgCmdArchivePush, argList);
 
                 s3 = storageRepoGet(0, true);
@@ -522,12 +534,12 @@ testRun(void)
                 TEST_ERROR_FMT(
                     storageGetP(storageNewReadP(s3, STRDEF("file.txt"))), ProtocolError,
                     "HTTP request failed with 301:\n"
-                        "*** Path/Query ***:\n"
-                        "GET /latest/meta-data/iam/security-credentials\n"
-                        "*** Request Headers ***:\n"
-                        "content-length: 0\n"
-                        "host: %s\n"
-                        "x-aws-ec2-metadata-token: WtokenW",
+                    "*** Path/Query ***:\n"
+                    "GET /latest/meta-data/iam/security-credentials\n"
+                    "*** Request Headers ***:\n"
+                    "content-length: 0\n"
+                    "host: %s\n"
+                    "x-aws-ec2-metadata-token: WtokenW",
                     strZ(hrnServerHost()));
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -549,7 +561,7 @@ testRun(void)
                 TEST_ERROR(
                     storageGetP(storageNewReadP(s3, STRDEF("file.txt"))), ProtocolError,
                     "role to retrieve temporary credentials not found\n"
-                        "HINT: is a valid IAM role associated with this instance?");
+                    "HINT: is a valid IAM role associated with this instance?");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("error when retrieving temp credentials and missing token");
@@ -576,11 +588,11 @@ testRun(void)
                 TEST_ERROR_FMT(
                     storageGetP(storageNewReadP(s3, STRDEF("file.txt"))), ProtocolError,
                     "HTTP request failed with 300:\n"
-                        "*** Path/Query ***:\n"
-                        "GET /latest/meta-data/iam/security-credentials/credrole\n"
-                        "*** Request Headers ***:\n"
-                        "content-length: 0\n"
-                        "host: %s",
+                    "*** Path/Query ***:\n"
+                    "GET /latest/meta-data/iam/security-credentials/credrole\n"
+                    "*** Request Headers ***:\n"
+                    "content-length: 0\n"
+                    "host: %s",
                     strZ(hrnServerHost()));
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -602,7 +614,7 @@ testRun(void)
                 TEST_ERROR_FMT(
                     storageGetP(storageNewReadP(s3, STRDEF("file.txt"))), ProtocolError,
                     "role '%s' not found\n"
-                        "HINT: is '%s' a valid IAM role associated with this instance?",
+                    "HINT: is '%s' a valid IAM role associated with this instance?",
                     strZ(credRole), strZ(credRole));
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -643,7 +655,7 @@ testRun(void)
                     auth,
                     .content = zNewFmt(
                         "{\"Code\":\"Success\",\"AccessKeyId\":\"x\",\"SecretAccessKey\":\"y\",\"Token\":\"z\""
-                            ",\"Expiration\":\"%s\"}",
+                        ",\"Expiration\":\"%s\"}",
                         strZ(testS3DateTime(time(NULL) + (S3_CREDENTIAL_RENEW_SEC - 1)))));
 
                 hrnServerScriptClose(auth);
@@ -694,14 +706,14 @@ testRun(void)
                     auth,
                     .content = zNewFmt(
                         "{\"Code\":\"Success\",\"AccessKeyId\":\"xx\",\"SecretAccessKey\":\"yy\",\"Token\":\"zz\""
-                            ",\"Expiration\":\"%s\"}",
+                        ",\"Expiration\":\"%s\"}",
                         strZ(testS3DateTime(time(NULL) + (S3_CREDENTIAL_RENEW_SEC * 2)))));
 
                 hrnServerScriptClose(auth);
 
                 testRequestP(
                     service, s3, HTTP_VERB_PUT, "/file.txt", .content = "ABCD", .accessKey = "xx", .securityToken = "zz",
-                    .kms = "kmskey1");
+                    .kms = "kmskey1", .tag = "%20Key%202=%20Value%202&Key1=Value1");
                 testResponseP(service);
 
                 // Make a copy of the signing key to verify that it gets changed when the keys are updated
@@ -733,7 +745,9 @@ testRun(void)
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("write zero-length file");
 
-                testRequestP(service, s3, HTTP_VERB_PUT, "/file.txt", .content = "", .kms = "kmskey1");
+                testRequestP(
+                    service, s3, HTTP_VERB_PUT, "/file.txt", .content = "", .kms = "kmskey1",
+                    .tag = "%20Key%202=%20Value%202&Key1=Value1");
                 testResponseP(service);
 
                 TEST_ASSIGN(write, storageNewWriteP(s3, STRDEF("file.txt")), "new write");
@@ -742,7 +756,9 @@ testRun(void)
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("write file in chunks with nothing left over on close");
 
-                testRequestP(service, s3, HTTP_VERB_POST, "/file.txt?uploads=", .kms = "kmskey1");
+                testRequestP(
+                    service, s3, HTTP_VERB_POST, "/file.txt?uploads=", .kms = "kmskey1",
+                    .tag = "%20Key%202=%20Value%202&Key1=Value1");
                 testResponseP(
                     service,
                     .content =
@@ -779,6 +795,9 @@ testRun(void)
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("error in success response of multipart upload");
 
+                // Stop writing tags
+                driver->tag = NULL;
+
                 testRequestP(service, s3, HTTP_VERB_POST, "/file.txt?uploads=", .kms = "kmskey1");
                 testResponseP(
                     service,
@@ -826,9 +845,9 @@ testRun(void)
                     "x-amz-security-token: <redacted>\n"
                     "*** Response Headers ***:\n"
                     "content-length: 110\n"
-                        "*** Response Content ***:\n"
+                    "*** Response Content ***:\n"
                     "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
-                        "<Error><Code>AccessDenied</Code><Message>Access Denied</Message></Error>");
+                    "<Error><Code>AccessDenied</Code><Message>Access Denied</Message></Error>");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("write file in chunks with something left over on close");
@@ -890,6 +909,7 @@ testRun(void)
                 #define TEST_SERVICE_URI                                                                                           \
                     "/?Action=AssumeRoleWithWebIdentity&RoleArn=arn%3Aaws%3Aiam%3A%3A123456789012%3Arole%2FTestRole"               \
                         "&RoleSessionName=pgBackRest&Version=2011-06-15&WebIdentityToken=" TEST_SERVICE_TOKEN
+                // {uncrustify_off - comment inside string}
                 #define TEST_SERVICE_RESPONSE                                                                                      \
                     "<AssumeRoleWithWebIdentityResponse xmlns=\"https://sts.amazonaws.com/doc/2011-06-15/\">\n"                    \
                     "  <AssumeRoleWithWebIdentityResult>\n"                                                                        \
@@ -901,6 +921,7 @@ testRun(void)
                     "    </Credentials>\n"                                                                                         \
                     "  </AssumeRoleWithWebIdentityResult>\n"                                                                       \
                     "</AssumeRoleWithWebIdentityResponse>"
+                // {uncrustify_on}
 
                 HRN_STORAGE_PUT_Z(storagePosixNewP(TEST_PATH_STR, .write = true), "web-id-token", TEST_SERVICE_TOKEN);
 
@@ -1015,7 +1036,8 @@ testRun(void)
                 testRequestP(service, s3, HTTP_VERB_GET, "/?delimiter=%2F&list-type=2");
                 testResponseP(service, .code = 344);
 
-                TEST_ERROR(storageListP(s3, STRDEF("/")), ProtocolError,
+                TEST_ERROR(
+                    storageListP(s3, STRDEF("/")), ProtocolError,
                     "HTTP request failed with 344:\n"
                     "*** Path/Query ***:\n"
                     "GET /?delimiter=%2F&list-type=2\n"
@@ -1039,7 +1061,8 @@ testRun(void)
                         "<Code>SomeOtherCode</Code>"
                         "</Error>");
 
-                TEST_ERROR(storageListP(s3, STRDEF("/")), ProtocolError,
+                TEST_ERROR(
+                    storageListP(s3, STRDEF("/")), ProtocolError,
                     "HTTP request failed with 344:\n"
                     "*** Path/Query ***:\n"
                     "GET /?delimiter=%2F&list-type=2\n"
@@ -1154,7 +1177,7 @@ testRun(void)
                 testRequestP(
                     service, s3, HTTP_VERB_GET,
                     "/?continuation-token=1ueGcxLPRx1Tr%2FXYExHnhbYLgveDs2J%2Fwm36Hy4vbOwM%3D&delimiter=%2F&list-type=2"
-                        "&prefix=path%2Fto%2F");
+                    "&prefix=path%2Fto%2F");
                 testResponseP(
                     service,
                     .content =
@@ -1370,8 +1393,8 @@ testRun(void)
                     .content =
                         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                         "<DeleteResult xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">"
-                            "<Error><Key>path/sample2.txt</Key><Code>AccessDenied</Code><Message>Access Denied</Message></Error>"
-                            "</DeleteResult>");
+                        "<Error><Key>path/sample2.txt</Key><Code>AccessDenied</Code><Message>Access Denied</Message></Error>"
+                        "</DeleteResult>");
 
                 testRequestP(service, s3, HTTP_VERB_DELETE, "/bucket/path/sample2.txt");
                 testResponseP(service, .code = 204);
